from insights.parsers.journalctl import JournalAll, JournalSinceBoot, JournalHeader
from insights.tests import context_wrap
from insights.parsers import journalctl
import doctest

JOURNAL_ALL_MSGINFO = """
-- Logs begin at Wed 2017-02-08 15:18:00 CET, end at Tue 2017-09-19 09:12:59 CEST. --
May 18 15:13:34 lxc-rhel68-sat56 jabberd/sm[11057]: session started: jid=rhn-dispatcher-sat@lxc-rhel6-sat56.redhat.com/superclient
May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: --> Wrapper Started as Daemon
May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM...
May 18 15:24:28 lxc-rhel68-sat56 yum[11597]: Installed: lynx-2.8.6-27.el6.x86_64
May 18 15:36:19 lxc-rhel68-sat56 yum[11954]: Updated: sos-3.2-40.el6.noarch
Apr 22 10:35:01 boy-bona CROND[27921]: (root) CMD (/usr/lib64/sa/sa1 -S DISK 1 1)
Apr 22 10:37:32 boy-bona crontab[28951]: (root) LIST (root)
Apr 22 10:40:01 boy-bona CROND[30677]: (root) CMD (/usr/lib64/sa/sa1 -S DISK 1 1)
Apr 22 10:41:13 boy-bona crontab[32515]: (root) LIST (root)
""".strip()

JOURNAL_SINCE_BOOT_MSGINFO = """
-- Logs begin at Wed 2017-02-08 15:18:00 CET, end at Tue 2017-09-19 09:12:59 CEST. --
May 18 15:13:34 lxc-rhel68-sat56 jabberd/sm[11057]: session started: jid=rhn-dispatcher-sat@lxc-rhel6-sat56.redhat.com/superclient
May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: --> Wrapper Started as Daemon
May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM...
May 18 15:24:28 lxc-rhel68-sat56 yum[11597]: Installed: lynx-2.8.6-27.el6.x86_64
May 18 15:36:19 lxc-rhel68-sat56 yum[11954]: Updated: sos-3.2-40.el6.noarch
Apr 22 10:35:01 boy-bona CROND[27921]: (root) CMD (/usr/lib64/sa/sa1 -S DISK 1 1)
Apr 22 10:37:32 boy-bona crontab[28951]: (root) LIST (root)
Apr 22 10:40:01 boy-bona CROND[30677]: (root) CMD (/usr/lib64/sa/sa1 -S DISK 1 1)
Apr 22 10:41:13 boy-bona crontab[32515]: (root) LIST (root)
""".strip()

JOURNALCTL_HEADER_VALID = """
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system.journal
File ID: b1390ea69aa747e9ac5c597835c3c562
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ONLINE
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 8388368
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: no
Head Sequential Number: 74388 (12294)
Tail Sequential Number: 81647 (13eef)
Head Realtime Timestamp: Mon 2022-08-15 12:01:10 CST (5e63fae9e6e58)
Tail Realtime Timestamp: Wed 2022-08-17 18:38:48 CST (5e66d7852bb3e)
Tail Monotonic Timestamp: 1month 2w 3d 14h 19min 3.733s (3c647d7ce0f)
Objects: 19073
Entry Objects: 7260
Data Objects: 9297
Data Hash Table Fill: 46.7%
Field Objects: 52
Field Hash Table Fill: 15.6%
Tag Objects: 0
Entry Array Objects: 2462
Disk usage: 8.0M
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system@435b0e30f47a46d8a2a2f9a42eae0aaf-000000000000f7d9-0005e5c6d743ca16.journal
File ID: 3aa38e09ef8540fdb55715e1739d44ef
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ARCHIVED
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 11464464
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: yes
Head Sequential Number: 63449 (f7d9)
Tail Sequential Number: 74387 (12293)
Head Realtime Timestamp: Tue 2022-08-09 11:51:01 CST (5e5c6d743ca16)
Tail Realtime Timestamp: Mon 2022-08-15 12:01:10 CST (5e63fae9e1ebf)
Tail Monotonic Timestamp: 1month 2w 1d 7h 41min 26.148s (3987e2331db)
Objects: 29134
Entry Objects: 10939
Data Objects: 14929
Data Hash Table Fill: 75.0%
Field Objects: 76
Field Hash Table Fill: 22.8%
Tag Objects: 0
Entry Array Objects: 3188
Disk usage: 10.9M
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system@435b0e30f47a46d8a2a2f9a42eae0aaf-000000000000cd85-0005e53f0e6e2342.journal
File ID: ab8fc76ad15b4c61bdc172533fe2e623
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ARCHIVED
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 11464464
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: yes
Head Sequential Number: 52613 (cd85)
Tail Sequential Number: 63448 (f7d8)
Head Realtime Timestamp: Tue 2022-08-02 17:51:11 CST (5e53f0e6e2342)
Tail Realtime Timestamp: Tue 2022-08-09 11:51:01 CST (5e5c6d7438053)
Tail Monotonic Timestamp: 1month 1w 2d 7h 31min 17.032s (31fa6c89323)
Objects: 29052
Entry Objects: 10836
Data Objects: 14929
Data Hash Table Fill: 75.0%
Field Objects: 50
Field Hash Table Fill: 15.0%
Tag Objects: 0
Entry Array Objects: 3235
Disk usage: 10.9M
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system@435b0e30f47a46d8a2a2f9a42eae0aaf-000000000000a315-0005e4ba61aa5231.journal
File ID: 428429bbac0c4051b0ece56a05ad2444
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ARCHIVED
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 11464464
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: yes
Head Sequential Number: 41749 (a315)
Tail Sequential Number: 52612 (cd84)
Head Realtime Timestamp: Wed 2022-07-27 03:33:57 CST (5e4ba61aa5231)
Tail Realtime Timestamp: Tue 2022-08-02 17:51:02 CST (5e53f0de06aa0)
Tail Monotonic Timestamp: 1month 2d 13h 31min 17.733s (297dd657d70)
Objects: 29018
Entry Objects: 10864
Data Objects: 14929
Data Hash Table Fill: 75.0%
Field Objects: 49
Field Hash Table Fill: 14.7%
Tag Objects: 0
Entry Array Objects: 3174
Disk usage: 10.9M
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system@435b0e30f47a46d8a2a2f9a42eae0aaf-000000000000785e-0005e437ff062b37.journal
File ID: d003176b94844c9bbada85c404d79c32
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ARCHIVED
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 11464464
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: yes
Head Sequential Number: 30814 (785e)
Tail Sequential Number: 41748 (a314)
Head Realtime Timestamp: Wed 2022-07-20 16:00:36 CST (5e437ff062b37)
Tail Realtime Timestamp: Wed 2022-07-27 03:33:56 CST (5e4ba61a1cb8a)
Tail Monotonic Timestamp: 3w 5d 9h 44min 12.265s (2133126de5b)
Objects: 29093
Entry Objects: 10935
Data Objects: 14929
Data Hash Table Fill: 75.0%
Field Objects: 49
Field Hash Table Fill: 14.7%
Tag Objects: 0
Entry Array Objects: 3178
Disk usage: 10.9M
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system@435b0e30f47a46d8a2a2f9a42eae0aaf-0000000000004ebd-0005e3aaad8b8fe3.journal
File ID: 9f5a3dbdf74343d88f78e4eb90864c01
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ARCHIVED
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 11464464
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: yes
Head Sequential Number: 20157 (4ebd)
Tail Sequential Number: 30813 (785d)
Head Realtime Timestamp: Wed 2022-07-13 15:24:39 CST (5e3aaad8b8fe3)
Tail Realtime Timestamp: Wed 2022-07-20 16:00:36 CST (5e437ff0629ba)
Tail Monotonic Timestamp: 2w 5d 22h 10min 52.149s (190ce8b3c8b)
Objects: 28782
Entry Objects: 10657
Data Objects: 14929
Data Hash Table Fill: 75.0%
Field Objects: 51
Field Hash Table Fill: 15.3%
Tag Objects: 0
Entry Array Objects: 3143
Disk usage: 10.9M
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system@435b0e30f47a46d8a2a2f9a42eae0aaf-0000000000002512-0005e333648ba707.journal
File ID: 9e1a381cedfa4a12a980872d6db84533
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ARCHIVED
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 11464464
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: yes
Head Sequential Number: 9490 (2512)
Tail Sequential Number: 20156 (4ebc)
Head Realtime Timestamp: Thu 2022-07-07 17:05:53 CST (5e333648ba707)
Tail Realtime Timestamp: Wed 2022-07-13 15:24:39 CST (5e3aaad8b8ea7)
Tail Monotonic Timestamp: 1w 5d 21h 34min 54.771s (1037d10a179)
Objects: 29077
Entry Objects: 10667
Data Objects: 14929
Data Hash Table Fill: 75.0%
Field Objects: 52
Field Hash Table Fill: 15.6%
Tag Objects: 0
Entry Array Objects: 3427
Disk usage: 10.9M
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system@435b0e30f47a46d8a2a2f9a42eae0aaf-0000000000000001-0005e2a7308e8200.journal
File ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ARCHIVED
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 11464464
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: yes
Head Sequential Number: 1 (1)
Tail Sequential Number: 9489 (2511)
Head Realtime Timestamp: Thu 2022-06-30 17:49:45 CST (5e2a7308e8200)
Tail Realtime Timestamp: Thu 2022-07-07 16:30:18 CST (5e332e54aafc7)
Tail Monotonic Timestamp: 6d 22h 40min 33.968s (8bb4cfc298)
Objects: 28018
Entry Objects: 9489
Data Objects: 14930
Data Hash Table Fill: 75.0%
Field Objects: 79
Field Hash Table Fill: 23.7%
Tag Objects: 0
Entry Array Objects: 3518
Disk usage: 10.9M
""".strip()

JOURNALCTL_HEADER_VALID_EXAMPLE = """
File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system.journal
File ID: b1390ea69aa747e9ac5c597835c3c562
Machine ID: 6bdaf92aa0754b53acbb1dbff7127e2b
Boot ID: 082ada53f8184c4896c73101ad793eb5
Sequential Number ID: 435b0e30f47a46d8a2a2f9a42eae0aaf
State: ONLINE
Compatible Flags:
Incompatible Flags: COMPRESSED-LZ4
Header size: 240
Arena size: 8388368
Data Hash Table Size: 19904
Field Hash Table Size: 333
Rotate Suggested: no
Head Sequential Number: 74388 (12294)
Tail Sequential Number: 81647 (13eef)
Head Realtime Timestamp: Mon 2022-08-15 12:01:10 CST (5e63fae9e6e58)
Tail Realtime Timestamp: Wed 2022-08-17 18:38:48 CST (5e66d7852bb3e)
Tail Monotonic Timestamp: 1month 2w 3d 14h 19min 3.733s (3c647d7ce0f)
Objects: 19073
Entry Objects: 7260
Data Objects: 9297
Data Hash Table Fill: 46.7%
Field Objects: 52
Field Hash Table Fill: 15.6%
Tag Objects: 0
Entry Array Objects: 2462
Disk usage: 8.0M
""".strip()


def test_journal_all_messages():
    msg_info = JournalAll(context_wrap(JOURNAL_ALL_MSGINFO))
    bona_list = msg_info.get('(root) LIST (root)')
    assert 2 == len(bona_list)
    assert bona_list[0].get('timestamp') == "Apr 22 10:37:32"
    assert bona_list[1].get('timestamp') == "Apr 22 10:41:13"
    crond = msg_info.get('CROND')
    assert 2 == len(crond)
    assert crond[0].get('procname') == "CROND[27921]"
    assert msg_info.get('jabberd/sm[11057]')[0].get('hostname') == "lxc-rhel68-sat56"
    assert msg_info.get('Wrapper')[0].get('message') == "--> Wrapper Started as Daemon"
    assert msg_info.get('Launching')[0].get('raw_message') == "May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM..."
    assert 2 == len(msg_info.get('yum'))


def test_journal_since_boot_messages():
    msg_info = JournalSinceBoot(context_wrap(JOURNAL_SINCE_BOOT_MSGINFO))
    bona_list = msg_info.get('(root) LIST (root)')
    assert 2 == len(bona_list)
    assert bona_list[0].get('timestamp') == "Apr 22 10:37:32"
    assert bona_list[1].get('timestamp') == "Apr 22 10:41:13"
    crond = msg_info.get('CROND')
    assert 2 == len(crond)
    assert crond[0].get('procname') == "CROND[27921]"
    assert msg_info.get('jabberd/sm[11057]')[0].get('hostname') == "lxc-rhel68-sat56"
    assert msg_info.get('Wrapper')[0].get('message') == "--> Wrapper Started as Daemon"
    assert msg_info.get('Launching')[0].get('raw_message') == "May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM..."
    assert 2 == len(msg_info.get('yum'))


def test_journalctl_header_valid():
    parser_result = JournalHeader(context_wrap(JOURNALCTL_HEADER_VALID))
    assert parser_result is not None
    result_list = parser_result.get('File Path')
    assert len(result_list) == 8
    assert result_list[0].get('raw_message') == 'File Path: /run/log/journal/6bdaf92aa0754b53acbb1dbff7127e2b/system.journal'


def test_losetup_doc_examples():
    env = {'journal_header': JournalHeader(context_wrap(JOURNALCTL_HEADER_VALID_EXAMPLE)),
           'JournalAll': JournalAll(context_wrap(JOURNAL_ALL_MSGINFO)),
           'JournalSinceBoot': JournalSinceBoot(context_wrap(JOURNAL_SINCE_BOOT_MSGINFO))}
    failed, total = doctest.testmod(journalctl, globs=env)
    assert failed == 0



"""
HponConf - command ``/sbin/hponcfg -g``
=======================================

Get the iLO firmware revision from the ``hponcfg`` command.
This is a 3rd party utility from HP and isn't shipped with RHEL.  However,
it's useful for detecting possible hardware incompatibilities.

There are only five pieces of information extracted:

* **firmware_revision** - the ``Firmware Revision`` value
* **device_type** - the ``Device type`` value
* **driver_name** - the ``Driver name`` value
* **server_name** - the ``Server Name`` value
* **server_number** - the ``Server Number`` value

Values are '' if not listed in the output.

Input looks like this::

    HP Lights-Out Online Configuration utility
    Version 4.3.1 Date 05/02/2014 (c) Hewlett-Packard Company, 2014
    Firmware Revision = 1.22 Device type = iLO 4 Driver name = hpilo
    Host Information:
                            Server Name: esxi01.hp.local
                            Server Number:

Examples:

    >>> cfg = shared[HponConf]
    >>> cfg.data['firmware_revision']
    '1.22'
    >>> cfg.data['server_name']
    'esxi01.hp.local'
    >>> cfg.data['server_number']
    ''
    >>> 'Version' in cfg.data # other values in the hponcfg output not found
    False
"""

from .. import parser, get_active_lines, CommandParser
from insights.specs import Specs

DRIVER_NAME = 'driver_name'
DEVICE_TYPE = 'device_type'
FIRMWARE_REVISION = 'firmware_revision'


@parser(Specs.hponcfg_g)
class HponConf(CommandParser):
    """
    Read the output of the HP ILO configuration utility.

    Attributes:
        firmware_revision (str): The firmware revision string.
        device_type (str): The device type (e.g. 'iLO 4').
        driver_name (str): The driver name (e.g. 'hpilo').
    """

    def parse_content(self, content):
        self.data = {}
        line_iter = iter(get_active_lines(content))
        while True:
            try:

                line = next(line_iter)

                if 'Firmware Revision' in line:
                    line = line.replace('Firmware Revision', '').replace('Device type', '').replace('Driver name', '')
                    val = [x.strip() for x in line.split('=') if x.strip()]
                    self.data[FIRMWARE_REVISION] = val[0]
                    self.data[DEVICE_TYPE] = val[1]
                    self.data[DRIVER_NAME] = val[2]

                if 'Host Information' in line:
                    line = next(line_iter).strip()
                    val = line.split('Server Name:')
                    if len(val) > 1:
                        self.data['server_name'] = val[1].strip()

                    line = next(line_iter).strip()
                    val = line.split('Server Number:')
                    if len(val) > 1:
                        self.data['server_number'] = val[1].strip()

            except StopIteration:
                break

    def __getitem__(self, i):
        return self.data[i]

    @property
    def firmware_revision(self):
        return self.data.get(FIRMWARE_REVISION, None)

    @property
    def device_type(self):
        return self.data.get(DEVICE_TYPE, None)

    @property
    def driver_name(self):
        return self.data.get(DRIVER_NAME, None)



from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.cmdline import CmdLine
from insights.parsers.systemd.unitfiles import ListUnits
from insights.parsers.redhat_release import RedhatRelease
from insights.combiners.rhel_for_edge import RhelForEdge
from insights.combiners import rhel_for_edge
from insights.tests import context_wrap
import doctest

CONTENT_REDHAT_RELEASE_RHEL = """
Red Hat Enterprise Linux release 8.4 (Ootpa)
""".strip()

CONTENT_REDHAT_RELEASE_COREOS = """
Red Hat Enterprise Linux CoreOS release 4.12
""".strip()

CMDLINE_RHEL = """
BOOT_IMAGE=/vmlinuz-4.18.0-80.el8.x86_64 root=/dev/mapper/rootvg-rootlv ro crashkernel=184M rd.lvm.lv=rootvg/rootlv biosdevname=0 printk.time=1 numa=off audit=1 transparent_hugepage=always LANG=en_US.UTF-8
""".strip()

CMDLINE_EDGE = """
BOOT_IMAGE=(hd0,msdos1)/ostree/rhel-323453435435234234232534534/vmlinuz-4.18.0-348.20.1.el8_5.x86_64 root=/dev/mapper/rhel-root ro crashkernel=auto rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap ostree=/ostree/boot.1/rhel/08987978979/0 rhgb quiet LANG=en_US.UTF-8
""".strip()

CONTENT_SYSTEMCTL_LIST_UNITS_NO_AUTOMATED = """
postgresql.service                                                                               loaded active running   PostgreSQL database server
""".strip()

CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED = """
postgresql.service                                                                               loaded active running   PostgreSQL database server
rhcd.service                                                                                     loaded active running   Red Hat connector daemon
""".strip()

CONTENT_INSTALLED_RPMS_RHEL = """
kernel-4.18.0-305.19.1.el8_4.x86_64
""".strip()

CONTENT_INSTALLED_RPMS_EDGE = """
kernel-4.18.0-305.19.1.el8_4.x86_64
rpm-ostree-2021.5-2.el8.x86_64
""".strip()


def test_rhel_for_edge_true_1():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_NO_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    result = RhelForEdge(install_rpms, cmdline, list_units, redhat_release)
    assert result.is_edge is True
    assert result.is_automated is False


def test_rhel_for_edge_true_2():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    result = RhelForEdge(install_rpms, cmdline, list_units, redhat_release)
    assert result.is_edge is True
    assert result.is_automated is True


def test_rhel_for_edge_false_1():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_RHEL))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    result = RhelForEdge(install_rpms, cmdline, list_units, redhat_release)
    assert result.is_edge is False
    assert result.is_automated is False


def test_rhel_for_edge_false_2():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_RHEL))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    result = RhelForEdge(install_rpms, cmdline, list_units, redhat_release)
    assert result.is_edge is False
    assert result.is_automated is False


def test_rhel_for_edge_false_3():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_COREOS))

    result = RhelForEdge(install_rpms, cmdline, list_units, redhat_release)
    assert result.is_edge is False
    assert result.is_automated is False


def test_doc_examples():
    install_rpms = InstalledRpms(context_wrap(CONTENT_INSTALLED_RPMS_EDGE))
    cmdline = CmdLine(context_wrap(CMDLINE_EDGE))
    list_units = ListUnits(context_wrap(CONTENT_SYSTEMCTL_LIST_UNITS_AUTOMATED))
    redhat_release = RedhatRelease(context_wrap(CONTENT_REDHAT_RELEASE_RHEL))

    env = {
            'rhel_for_edge_obj': RhelForEdge(install_rpms, cmdline, list_units, redhat_release)
          }
    failed, total = doctest.testmod(rhel_for_edge, globs=env)
    assert failed == 0



import doctest
from insights.parsers import dnf_modules
from insights.tests import context_wrap

DNF_MODULES_INPUT = """
[postgresql]
name=postgresql
profiles=client
state=enabled
stream=9.6
[python36]
name=python36
profiles=
state=enabled
stream=3.6
[virt]
name=virt
profiles=
state=enabled
stream=rhel
"""


def test_dnf_modules():
    modules_config = dnf_modules.DnfModules(context_wrap(DNF_MODULES_INPUT))
    assert modules_config is not None
    assert 'postgresql' in modules_config.sections()
    assert 'python36' in modules_config.sections()
    assert 'virt' in modules_config.sections()
    assert 'enabled' == modules_config.get('postgresql', 'state')


def test_dnf_modules_doc_examples():
    failed, total = doctest.testmod(
        dnf_modules,
        globs={'dnf_modules': dnf_modules.DnfModules(context_wrap(DNF_MODULES_INPUT))}
    )
    assert failed == 0



from insights import combiner
from insights.tests import archive_provider, InputData


@combiner()
def one():
    return 1


@combiner()
def two():
    return 2


@combiner(one, two)
def three(x, y):
    return x + y


@archive_provider(three)
def integration_tests():
    data = InputData()
    data.add_component(two, 5)
    yield data, 6



"""
rhn_schema_version - Command ``/usr/bin/rhn-schema-version``
============================================================
Parse the output of command ``/usr/bin/rhn-schema-version``.

"""
from .. import parser
from insights.specs import Specs


@parser(Specs.rhn_schema_version)
def rhn_schema_version(context):
    """
    Function to parse the output of command ``/usr/bin/rhn-schema-version``.

    Sample input::

        5.6.0.10-2.el6sat

    Examples:
        >>> db_ver = shared[rhn_schema_version]
        >>> db_ver
        '5.6.0.10-2.el6sat'

    """
    if context.content:
        content = context.content
        if len(content) == 1 and 'No such' not in content[0]:
            ver = content[0].strip()
            if ver:
                return ver



"""
HammerPing - command ``/usr/bin/hammer ping``
=============================================

The hammer ping parser reads the output of ``hammer ping`` and turns it into
a dictionary. The key is the service name, and the value is a dict of all the
service info.

Sample output of ``hammer ping``::

    candlepin:
        Status:          FAIL
        Server Response: Message: 404 Resource Not Found
    elasticsearch:
        Status:          ok
        Server Response: Duration: 35ms
    foreman_tasks:
        Status:          ok
        Server Response: Duration: 1ms

Examples:

    >>> type(hammer_ping)
    <class 'insights.parsers.hammer_ping.HammerPing'>
    >>> 'unknown_service' in hammer_ping.service_list
    False
    >>> hammer_ping['candlepin']['Status']
    'FAIL'
    >>> hammer_ping['candlepin']['Server Response']
    'Message: 404 Resource Not Found'
    >>> hammer_ping.are_all_ok
    False
    >>> hammer_ping.services_of_status('OK')
    ['elasticsearch', 'foreman_tasks']
"""
from insights import parser, CommandParser
from insights.specs import Specs
from insights.parsers import SkipException


@parser(Specs.hammer_ping)
class HammerPing(CommandParser, dict):
    """
    Read the ``hammer ping`` status and convert it to dictionaries of
    status and response information.

    Attributes:
        errors (list): Any error messages encountered during parsing
        raw_content(list): The original output of hammer ping
    """

    def services_of_status(self, status='ok'):
        """
        List of the services in the given status.

        Arguments:
            status (str): the status code to search for, defaulting to 'ok'.
                The value is converted to lower case.

        Returns: List of service names having that status.
        """
        return sorted([svc for svc in self if self[svc]['Status'].lower() == status.lower()])

    @property
    def service_list(self):
        """Return a list of service in order """
        return sorted(self.keys())

    @property
    def are_all_ok(self):
        """Return boolean value to indicate if all the service are running normally"""
        return self._is_normal

    def parse_content(self, content):
        self.status_of_service = {}
        self.response_of_service = {}
        self.errors = []
        self.raw_content = content
        comment_char = "COMMAND>"
        service_name = None
        content = list(filter(None, (line.split(comment_char, 1)[0].rstrip() for line in content)))
        if not content:
            raise SkipException("Empty output.")

        for line in content:
            items = [item for item in line.split(':', 1)]
            if len(items) == 2 and items[0]:
                if not items[1] and len(items[0].split()) == 1 and not items[0][0].isspace():
                    service_name = items[0].strip()
                    self[service_name] = {}
                    continue
                elif service_name is not None and items[0][0].isspace():
                    items[0] = items[0].strip()
                    items[1] = items[1].strip()
                    self[service_name][items[0]] = items[1]
                    if items[0] == 'Status':
                        self.status_of_service[service_name] = items[1].lower()
                    if items[0] == 'Server Response':
                        self.response_of_service[service_name] = items[1]
                    continue

            self.errors.append(line.strip())
        self._is_normal = (not self.errors and all([self[item]['Status'] == 'ok' for item in self]))



from insights.parsers.rhn_conf import RHNConf
from insights.tests import context_wrap

RHN_TEST = """
a=1
b = 2

#c = 3
c = include an = sign
server.satellite.http_proxy = corporate_gateway.example.com:8080
server.satellite.http_proxy_username =
server.satellite.http_proxy_password =

traceback_mail = test@example.com, test@redhat.com

web.default_taskmaster_tasks = RHN::Task::SessionCleanup, RHN::Task::ErrataQueue,
    RHN::Task::ErrataEngine,
    RHN::Task::DailySummary, RHN::Task::SummaryPopulation,
    RHN::Task::RHNProc,
    RHN::Task::PackageCleanup

db_host =
ignored
""".strip()


def test_rhn_conf():
    r = RHNConf(context_wrap(RHN_TEST))
    assert r["a"] == "1"
    assert r["b"] == "2"
    assert r["c"] == "include an = sign"
    assert r["server.satellite.http_proxy_username"] == ""
    assert r["traceback_mail"] == ['test@example.com', 'test@redhat.com']
    assert r["web.default_taskmaster_tasks"] == [
        'RHN::Task::SessionCleanup', 'RHN::Task::ErrataQueue',
        'RHN::Task::ErrataEngine', 'RHN::Task::DailySummary',
        'RHN::Task::SummaryPopulation', 'RHN::Task::RHNProc',
        'RHN::Task::PackageCleanup'
    ]



from collections import namedtuple
from contextlib import contextmanager
from json import dump
from os import mkdir
from os.path import join
from shutil import rmtree
from tempfile import mkdtemp

from mock.mock import call
from mock.mock import Mock
from mock.mock import patch
from pytest import mark

from insights.client.config import InsightsConfig
from insights.client.data_collector import CleanOptions
from insights.contrib.soscleaner import SOSCleaner


class ReportFile:
    def __init__(self, report_item):
        self.report_item = report_item

    @staticmethod
    def result(relative_path):
        return {
            "object": {
                "relative_path": relative_path
            }
        }

    @property
    def file_name(self):
        return "%s.json" % self.report_item.name

    @property
    def contents(self):
        return {
            "name": self.report_item.name,
            "results": self.report_item.results,
        }


class ReportItemSimpleFile(namedtuple("ReportItemSimpleFile", ("name", "relative_path"))):
    @property
    def results(self):
        return ReportFile.result(self.relative_path)

    @property
    def relative_paths(self):
        return [self.relative_path]


class ReportItemGlobFile(namedtuple("ReportItemGlobFile", ("name", "relative_paths"))):
    @property
    def results(self):
        return [ReportFile.result(relative_path) for relative_path in self.relative_paths]


def _soscleaner():
    soscleaner = SOSCleaner()
    soscleaner.logger = Mock()
    return soscleaner


@contextmanager
def _mock_report_dir(report_items):
    tmpdir = mkdtemp()
    try:
        hostname_path = join(tmpdir, "hostname")
        with open(hostname_path, "w") as hostname_file:
            hostname_file.write("test-hostname")

        data_path = join(tmpdir, "data")
        meta_data_path = join(tmpdir, "meta_data")
        for path in [data_path, meta_data_path]:
            mkdir(path)

        for report_item in report_items:
            report_file = ReportFile(report_item)
            json_path = join(meta_data_path, report_file.file_name)
            with open(json_path, "w") as json_file:
                meta_data_obj = report_file.contents
                dump(meta_data_obj, json_file)

        yield tmpdir
    finally:
        rmtree(tmpdir)


@mark.parametrize(("line", "expected"), [
    ("radius_ip_1=10.0.0.1", "radius_ip_1=10.230.230.1"),
    (
        (
            "        inet 10.0.2.15"
            "  netmask 255.255.255.0"
            "  broadcast 10.0.2.255"
        ),
        (
            "        inet 10.230.230.3"
            "  netmask 10.230.230.1"
            "  broadcast 10.230.230.2"
        ),
    ),
    (
        "radius_ip_1=10.0.0.100-10.0.0.200",
        "radius_ip_1=10.230.230.1-10.230.230.2",
    ),
])
def test_sub_ip_match(line, expected):
    soscleaner = _soscleaner()
    actual = soscleaner._sub_ip(line)
    assert actual == expected


@mark.parametrize(("line", "expected"), [
    (
        (
            "        inet 10.0.2.155"
            "  netmask 10.0.2.1"
            "  broadcast 10.0.2.15"
        ),
        (
            "        inet 10.230.230.1"
            "  netmask 10.230.230.3"
            "  broadcast 10.230.230.2"
        ),
    ),
])
def test_sub_ip_match_IP_overlap(line, expected):
    soscleaner = _soscleaner()
    actual = soscleaner._sub_ip(line)
    assert actual == expected


@mark.parametrize(("line", "expected"), [
    (
        "tcp6       0      0 10.0.0.1:23           10.0.0.110:63564   ESTABLISHED 0",
        "tcp6       0      0 10.230.230.2:23       10.230.230.1:63564 ESTABLISHED 0"
    ),
    (
        "tcp6  10.0.0.11    0 10.0.0.1:23       10.0.0.111:63564    ESTABLISHED 0",
        "tcp6  10.230.230.2 0 10.230.230.3:23   10.230.230.1:63564  ESTABLISHED 0"
    ),
    (
        "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         172.31.0.1\n",
        "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         10.230.230.1\n"
    ),
    (
        "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         172.31.111.11\n",
        "unix  2      [ ACC ]     STREAM     LISTENING     43279    2070/snmpd         10.230.230.1 \n"
    ),
])
def test_sub_ip_match_IP_overlap_netstat(line, expected):
    soscleaner = _soscleaner()
    actual = soscleaner._sub_ip_netstat(line)
    assert actual == expected


@mark.parametrize(("original", "expected"), [
    (
        "{\"name\":\"shadow-utils\","
        "\"epoch\":\"2\","
        "\"version\":\"4.1.5.1\","
        "\"release\":\"5.el6\","
        "\"arch\":\"x86_64\","
        "\"installtime\":\"Wed 13 Jan 2021 10:04:18 AM CET\","
        "\"buildtime\":\"1455012203\","
        "\"vendor\":\"Red Hat, Inc.\","
        "\"buildhost\":\"x86-027.build.eng.bos.redhat.com\","
        "\"sigpgp\":"
        "\"RSA/8, "
        "Tue 08 Mar 2016 11:15:08 AM CET, "
        "Key ID 199e2f91fd431d51\"}",

        "{\"name\":\"shadow-utils\","
        "\"epoch\":\"2\","
        "\"version\":\"10.230.230.1\","
        "\"release\":\"5.el6\","
        "\"arch\":\"x86_64\","
        "\"installtime\":\"Wed 13 Jan 2021 10:04:18 AM CET\","
        "\"buildtime\":\"1455012203\","
        "\"vendor\":\"Red Hat, Inc.\","
        "\"buildhost\":\"x86-027.build.eng.bos.redhat.com\","
        "\"sigpgp\":"
        "\"RSA/8, "
        "Tue 08 Mar 2016 11:15:08 AM CET, "
        "Key ID 199e2f91fd431d51\"}",
    )
])
@patch("insights.contrib.soscleaner.SOSCleaner._ip2db", return_value="10.230.230.1")
def test_sub_ip_false_positive(_ip2db, original, expected):
    soscleaner = _soscleaner()
    actual = soscleaner._sub_ip(original)
    assert actual == expected


def test_excluded_specs():
    soscleaner = SOSCleaner()
    assert len(soscleaner.excluded_specs) > 0
    for spec in soscleaner.excluded_specs:
        assert spec.find("insights.specs.Specs.") == 0


def test_excluded_files_simple_file():
    report_items = [
        ReportItemSimpleFile(
            "insights.specs.Specs.installed_rpms",
            "insights_commands/rpm_-qa_--qf_name_NAME_epoch_EPOCH_"
            "version_VERSION_release_RELEASE_arch_ARCH_"
            "installtime_INSTALLTIME_date_buildtime_BUILDTIME_"
            "vendor_VENDOR_buildhost_BUILDHOST_sigpgp_SIGPGP_pgpsig"
        ),
        ReportItemSimpleFile(
            "insights.specs.Specs.ip_addr",
            "insights_commands/ip_addr"
        )
    ]

    soscleaner = _soscleaner()
    with _mock_report_dir(report_items) as tmpdir:
        soscleaner.dir_path = tmpdir
        actual = soscleaner._excluded_files()

    expected = report_items[0].relative_paths
    assert actual == expected


def test_excluded_files_glob_file():
    report_items = [
        ReportItemGlobFile(
            "insights.specs.Specs.dnf_modules",
            ["etc/dnf/modules.d/ruby.module"]
        ),
        ReportItemGlobFile(
            "insights.specs.Specs.dnsmasq_config",
            ["etc/dnsmasq.conf"]
        )
    ]

    soscleaner = _soscleaner()
    with _mock_report_dir(report_items) as tmpdir:
        soscleaner.dir_path = tmpdir
        actual = soscleaner._excluded_files()

    assert actual == report_items[0].relative_paths


@patch("insights.contrib.soscleaner.SOSCleaner._clean_file")
@patch("insights.contrib.soscleaner.SOSCleaner._excluded_files")
@patch("insights.contrib.soscleaner.SOSCleaner._extract_sosreport")
@patch("insights.contrib.soscleaner.SOSCleaner._make_dest_env")
@patch("insights.contrib.soscleaner.SOSCleaner._start_logging")
def test_clean_report(
    _start_logging,
    _make_dest_env,
    _extract_sosreport,
    excluded_files,
    clean_file
):
    class FileList:
        def __init__(self, soscleaner):
            self.soscleaner = soscleaner

        def __call__(self, data_path):
            self.soscleaner.file_count = len(file_list_files)

            paths = []
            for file in file_list_files:
                path = join(self.soscleaner.dir_path, data_path, file)
                paths.append(path)
            self.return_value = paths
            return paths

    file_list_files = [
        "insights_commands/rpm_-qa_--qf_name_NAME_version_VERSION",
        "insights_commands/ip_addr"
    ]

    file_list_exempt = file_list_files[0:1]
    excluded_files.configure_mock(return_value=file_list_exempt)

    config = InsightsConfig()
    options = CleanOptions(config, None, None, None)
    options.no_tar_file = "."

    soscleaner = _soscleaner()
    with patch(
        "insights.contrib.soscleaner.SOSCleaner._file_list",
        FileList(soscleaner)
    ) as file_list:
        soscleaner.clean_report(options, ".")

    file_list_whitelisted = file_list.return_value[1:2]
    calls_whitelisted = map(call, file_list_whitelisted)
    assert clean_file.mock_calls == list(calls_whitelisted)



"""
Open vSwitch ``ovs-vsctl`` - utility for querying ovs-vswitchd
==============================================================

Classes in this module are:

OVSvsctlList - command ``/usr/bin/ovs-vsctl list TBL [REC]``
------------------------------------------------------------

Parsers in this module are:

OVSvsctlListBridge - command ``/usr/bin/ovs-vsctl list bridge``
---------------------------------------------------------------
"""

from insights import CommandParser, get_active_lines, parser
from insights.parsers import SkipException, optlist_to_dict
from insights.specs import Specs


class OVSvsctlList(CommandParser, list):
    """
    Class to parse output of command ``ovs-vsctl list TBL [REC]``.
    Generally, the data is in ``key:value`` format with values having
    data types as string, numbers, list or dictionary.

    Raises:
        SkipException: When file is empty.
    """
    def parse_content(self, content):
        """
           Details of the subset of the Open vSwitch database, which holds the configuration
           for the Open vSwitch daemon, are extracted and stored in a list as a dictionary.
        """
        # No content found or file is empty
        if not content:
            raise SkipException("Empty file")

        record = {}
        for line in get_active_lines(content):
            key, value = [i.strip() for i in line.split(":", 1)]
            parsed_value = value.strip('"')
            if value.startswith("{") and value.endswith("}"):
                parsed_value = {}
                value = value.strip("{}")
                if value:
                    parsed_value = optlist_to_dict(value, opt_sep=", ", strip_quotes=True)
            elif value.startswith("[") and value.endswith("]"):
                parsed_value = []
                value = value.strip("[]")
                if value:
                    parsed_value = [i.strip(' \t\"\'') for i in value.split(",")]

            if key in record:
                # A new record comes
                self.append(record)
                record = {}

            record[key] = parsed_value

        # Add the last record
        self.append(record)

    @property
    def data(self):
        """
        Set data as property to keep compatibility
        """
        return self


@parser(Specs.ovs_vsctl_list_bridge)
class OVSvsctlListBridge(OVSvsctlList):
    """
    Class to parse output of command ``ovs-vsctl list bridge``.

    Sample command output::

        name                : br-int
        other_config        : {disable-in-band="true", mac-table-size="2048"}
        name                : br-tun
        other_config        : {}

    Examples:
        >>> bridge_lists[0]["name"]
        'br-int'
        >>> bridge_lists[0]["other_config"]["mac-table-size"]
        '2048'
        >>> bridge_lists[0]["other_config"]["disable-in-band"]
        'true'
        >>> bridge_lists[1].get("name")
        'br-tun'
        >>> len(bridge_lists[1]["other_config"]) == 0
        True
    """
    pass



from insights.parsers.tmpfilesd import TmpFilesD
from insights.tests import context_wrap

SAP_CONF = """
# systemd tmpfiles exclude file for SAP
# SAP software stores some important files
# in /tmp which should not be deleted

# Exclude SAP socket and lock files
x /tmp/.sap*

# Exclude HANA lock file
x /tmp/.hdb*lock
""".strip()


def test_tmpfilesd():
    ctx = context_wrap(SAP_CONF, path='/etc/tmpfiles.d/sap.conf')
    data = TmpFilesD(ctx)

    assert len(data.files) == 2
    assert data.files == ['/tmp/.sap*', '/tmp/.hdb*lock']
    assert data.rules == [{'type': 'x',
                           'mode': None,
                           'path': '/tmp/.sap*',
                           'uid': None,
                           'gid': None,
                           'age': None,
                           'argument': None},
                          {'type': 'x',
                           'path': '/tmp/.hdb*lock',
                           'mode': None,
                           'uid': None,
                           'gid': None,
                           'age': None,
                           'argument': None}]
    assert data.file_path == '/etc/tmpfiles.d/sap.conf'
    assert data.file_name == 'sap.conf'


def test_find_file():
    ctx = context_wrap(SAP_CONF, path='/etc/tmpfiles.d/sap.conf')
    data = TmpFilesD(ctx)

    assert data.find_file('.sap*') == [{'path': '/tmp/.sap*', 'type': 'x', 'mode': None,
                                        'age': None, 'gid': None, 'uid': None,
                                        'argument': None}]
    assert data.find_file('.hdb*lock') == [{'path': '/tmp/.hdb*lock', 'type': 'x',
                                            'mode': None, 'uid': None, 'gid': None,
                                            'age': None, 'argument': None}]
    assert data.find_file('bar') == []



from insights.parsers.hammer_ping import HammerPing
from insights.tests import context_wrap
from insights.parsers import SkipException, hammer_ping
import doctest
import pytest

HAMMERPING_ERR_1 = """
Error: Connection refused - connect(2) for "localhost" port 443
"""

HAMMERPING_ERR_2 = """
Could not load the API description from the server
 - is the server down?
 - was 'foreman-rake apipie:cache' run on the server when using apipie cache? (typical production settings)
Warning: An error occured while loading module hammer_cli_csv
Could not load the API description from the server
 - is the server down?
 - was 'foreman-rake apipie:cache' run on the server when using apipie cache? (typical production settings)
Warning: An error occured while loading module hammer_cli_foreman
"""

HAMMERPING_OK_1 = """
candlepin:
    Status:          ok
    Server Response: Duration: 61ms
candlepin_auth:
    Status:          ok
"""

HAMMERPING_ERR_3 = """
database:
   Status:          ok
   Server Response: Duration: 0ms
katello_agent:
   Status:          ok
   message:         0 Processed, 0 Failed
   Server Response: Duration: 0ms
candlepin:
   Status:          ok
   Server Response: Duration: 45ms
candlepin_auth:
   Status:          ok
   Server Response: Duration: 45ms
candlepin_events:
   Status:          ok
   message:         0 Processed, 0 Failed
   Server Response: Duration: 0ms
katello_events:
   Status:          ok
   message:         0 Processed, 0 Failed
   Server Response: Duration: 0ms
pulp3:
   Status:          ok
   Server Response: Duration: 294ms
pulp3_content:
   Status:          ok
   Server Response: Duration: 58ms
foreman_tasks:
   Status:          ok
   Server Response: Duration: 3ms

2 more service(s) failed, but not shown:
pulp, pulp_auth
"""

HAMMERPING_OK = """
candlepin:
    Status:          ok
    Server Response: Duration: 61ms
candlepin_auth:
    Status:          ok
    Server Response: Duration: 61ms
pulp:
    Status:          ok
    Server Response: Duration: 61ms
pulp_auth:
    Status:          ok
    Server Response: Duration: 61ms
elasticsearch:
    Status:          ok
    Server Response: Duration: 35ms
foreman_tasks:
    Status:          ok
    server Response: Duration: 1ms
""".strip()

HAMMERPING_COMMAND = """
COMMAND> hammer ping

candlepin:
    Status:          ok
    Server Response: Duration: 20ms
candlepin_auth:
    Status:          ok
    Server Response: Duration: 14ms
pulp:
    Status:          ok
    Server Response: Duration: 101ms
pulp_auth:
    Status:          ok
    Server Response: Duration: 75ms
foreman_tasks:
    Status:          ok
    Server Response: Duration: 3ms

""".strip()

HAMMERPING = """
candlepin:
    Status:          FAIL
    Server Response:Message:404 Resource Not Found
candlepin_auth:
    Status:          FAIL
    Server Response: Message: Katello::Resources::Candlepin::CandlepinPing: 404 Resource Not Found
pulp:
    Status:          ok
    Server Response: Duration: 61ms
pulp_auth:
    Status:
    Server Response:
elasticsearch:
    Status:          ok
    Server Response: Duration: 35ms
foreman_tasks:
    Status:          ok
    server Response: Duration: 1ms
""".strip()

HAMMERPING_FAIL = """
candlepin:
    Status:          ok
    Server Response:
candlepin_auth:
    Status:          ok
    Server Response:
pulp:
    Status:          FAIL
    Server Response:
pulp_auth:
    Status: FAIL
foreman_tasks:
    Status:          ok
    Server Response: Duration: 28ms
""".strip()

HAMMERPING_EXAMPLE = """
candlepin:
    Status:          FAIL
    Server Response: Message: 404 Resource Not Found
elasticsearch:
    Status:          ok
    Server Response: Duration: 35ms
foreman_tasks:
    Status:          ok
    Server Response: Duration: 1ms
""".strip()

HAMMERPING_EMPTY = """
COMMAND> hammer ping

""".strip()


def test_hammer_ping_err_1():
    status = HammerPing(context_wrap(HAMMERPING_ERR_1))
    assert not status.are_all_ok
    assert status.errors != []


def test_hammer_ping_err_2():
    status = HammerPing(context_wrap(HAMMERPING_ERR_2))
    assert not status.are_all_ok
    assert status.errors != []


def test_hammer_ping_err_3():
    status = HammerPing(context_wrap(HAMMERPING_OK_1))
    assert status.are_all_ok
    assert status.errors == []


def test_hammer_ping_err_4():
    status = HammerPing(context_wrap(HAMMERPING_ERR_3))
    assert not status.are_all_ok
    assert status.errors != []


def test_hammer_ping_ok():
    status = HammerPing(context_wrap(HAMMERPING_OK))

    assert status.are_all_ok
    assert status.service_list == [
        'candlepin', 'candlepin_auth', 'elasticsearch', 'foreman_tasks', 'pulp', 'pulp_auth',
    ]
    assert status.services_of_status('FAIL') == []
    assert 'nonexistent' not in status.service_list


def test_hammer_ping_command():
    status = HammerPing(context_wrap(HAMMERPING_COMMAND))

    assert status.are_all_ok
    assert status.service_list == [
        'candlepin', 'candlepin_auth', 'foreman_tasks', 'pulp', 'pulp_auth',
    ]
    assert status.services_of_status('FAIL') == []
    assert 'nonexistent' not in status.service_list


def test_hammer_ping():
    status = HammerPing(context_wrap(HAMMERPING))

    assert not status.are_all_ok
    assert status.service_list == [
        'candlepin', 'candlepin_auth', 'elasticsearch', 'foreman_tasks',
        'pulp', 'pulp_auth'
    ]
    assert status.services_of_status('OK') == [
        'elasticsearch', 'foreman_tasks', 'pulp'
    ]
    assert status.services_of_status('FAIL') == [
        'candlepin', 'candlepin_auth'
    ]
    assert status['pulp_auth']['Status'] == ''
    assert status['candlepin']['Status'] == 'FAIL'
    assert status['elasticsearch']['Status'] == 'ok'
    assert status['pulp_auth']['Server Response'] == ''
    assert status['candlepin_auth']['Server Response'] == 'Message: Katello::Resources::Candlepin::CandlepinPing: 404 Resource Not Found'
    assert status['elasticsearch']['Server Response'] == 'Duration: 35ms'

    assert 'nonexistent' not in status.service_list
    assert 'nonexistent' not in status


def test_hammer_different_lines():
    status = HammerPing(context_wrap(HAMMERPING_FAIL))
    assert status.services_of_status('FAIL') == [
        'pulp', 'pulp_auth'
    ]

    assert status.services_of_status('ok') == [
        'candlepin', 'candlepin_auth', 'foreman_tasks'
    ]


def test_status_and_response():
    status = HammerPing(context_wrap(HAMMERPING_FAIL))
    assert status.status_of_service['pulp'] == 'fail'
    assert status.status_of_service['foreman_tasks'] == 'ok'

    assert status.response_of_service['pulp'] == ''
    assert status.response_of_service['foreman_tasks'] == 'Duration: 28ms'


def test_raw_content():
    status = HammerPing(context_wrap(HAMMERPING_COMMAND))
    for line in HAMMERPING_COMMAND.splitlines():
        assert line in status.raw_content


def test_content_empty():
    with pytest.raises(SkipException):
        HammerPing(context_wrap(HAMMERPING_EMPTY))


def test_losetup_doc_examples():
    env = {'hammer_ping': HammerPing(context_wrap(HAMMERPING_EXAMPLE))}
    failed, total = doctest.testmod(hammer_ping, globs=env)
    assert failed == 0



from insights.core import dr
from insights.core.plugins import datasource, rule, make_info, ContentException
from insights.core.spec_factory import RegistryPoint, SpecSet


class Specs(SpecSet):
    the_data = RegistryPoint()


class TestSpecs(Specs):
    @datasource()
    def the_data(broker):
        raise ContentException('Fake Datasource')


@rule(Specs.the_data)
def report(dt):
    return make_info('INFO_1')


def test_broker_add_exception():
    broker = dr.run(report)
    assert report in broker
    assert TestSpecs.the_data in broker.exceptions
    spec_exs = broker.exceptions[TestSpecs.the_data]
    exs = [ex for ex in spec_exs if isinstance(ex, ContentException) and str(ex) == "Fake Datasource"]
    assert len(exs) == 1
    tb = broker.tracebacks[exs[0]]
    assert type(tb) is str
    assert "Traceback" in tb
    assert "Fake Datasource" in tb



from insights.parsers.getconf_pagesize import GetconfPageSize
from insights.tests import context_wrap
from insights.parsers import getconf_pagesize
import doctest

GETCONFPAGESIZE1 = """
4096
""".strip()

GETCONFPAGESIZE2 = """
16384
""".strip()


def test_getconf_PAGESIZE1():
    result = GetconfPageSize(context_wrap(GETCONFPAGESIZE1))
    assert result.page_size == 4096


def test_getconf_PAGESIZE2():
    result = GetconfPageSize(context_wrap(GETCONFPAGESIZE2))
    assert result.page_size == 16384


def test_doc():
    env = {
            "pagesize_parsed": GetconfPageSize(context_wrap(GETCONFPAGESIZE1))
    }
    print(env)
    failed, total = doctest.testmod(getconf_pagesize, globs=env)
    assert failed == 0



from insights.parsers.crontab import CrontabL, HeatCrontab, KeystoneCrontab, NovaCrontab, RootCrontab
from insights.tests import context_wrap
from insights.parsers import ParseException

import pytest

CRONTAB_DATA = """
# comment line
* * * * * /usr/bin/keystone-manage token_flush > /dev/null 2>&1
33 0 1 3 6 /bin/heat-manage purge_deleted -g days 7
# Test all the range and list and named options
  00-59/20  1,2,3,5,8,13,21 */2 1,MAR,7-Dec/2   Tue,4-Sun   /bin/mail -s "Foo has occurred" admin%Foo has occurred, time to bail out
* * * * * user_name /usr/local/bin/motd_update
MAILTO=admin
CRON_TZ = Europe/France
"""

CRONTAB_EXTENSIONS = """
@reboot /usr/local/bin/unmangle_database
@daily  /usr/sbin/logrotate
"""

CRONTAB_BAD_DATA = """
* * * *  /usr/bin/heat-manage purge_deleted >/dev/null 2>&1
Error reading crontab
"""

CRONTAB_BAD_NICKNAME = """
@huorly  /usr/bin/check_init
"""


def crontab_tests(crontab_class):
    crontab = crontab_class(context_wrap(CRONTAB_DATA))
    assert crontab is not None
    assert [r.get('minute') for r in crontab] == ['*', '33', '00-59/20', '*']
    assert crontab.search('heat-manage') == [
        {'minute': '33',
         'hour': '0',
         'day_of_month': '1',
         'month': '3',
         'day_of_week': '6',
         'command': '/bin/heat-manage purge_deleted -g days 7'}]
    assert crontab.search('crazy_123') == []
    assert crontab.search('motd')[0]['command'] == 'user_name /usr/local/bin/motd_update'
    assert crontab.invalid_lines == []

    # Test environment variable handling
    assert hasattr(crontab, 'environment')
    assert isinstance(crontab.environment, dict)
    assert 'MAILTO' in crontab.environment
    assert crontab.environment['MAILTO'] == 'admin'
    assert 'CRON_TZ' in crontab.environment
    assert crontab.environment['CRON_TZ'] == 'Europe/France'

    # Test that bad data gets stored in the invalid_lines property
    crontab = crontab_class(context_wrap(CRONTAB_DATA + CRONTAB_BAD_DATA))
    assert crontab is not None
    assert crontab.invalid_lines == [
        '* * * *  /usr/bin/heat-manage purge_deleted >/dev/null 2>&1',
        'Error reading crontab',
    ]

    # Test that crontab 'special time nicknames' are parsed correctly
    crontab = crontab_class(context_wrap(CRONTAB_EXTENSIONS))
    assert crontab is not None
    assert crontab.invalid_lines == []
    assert crontab[0] == {
        'time': '@reboot',
        'command': '/usr/local/bin/unmangle_database',
    }
    assert crontab[1] == {
        'minute': '0',
        'hour': '0',
        'day_of_month': '*',
        'month': '*',
        'day_of_week': '*',
        'command': '/usr/sbin/logrotate',
    }

    # Test that crontab 'special names' only works with known names
    with pytest.raises(ParseException) as exc:
        crontab = crontab_class(context_wrap(CRONTAB_EXTENSIONS + CRONTAB_BAD_NICKNAME))
    assert "@huorly not recognised as a time specification 'nickname'" in str(exc)

    # Test that crontab 'special time nicknames' are parsed correctly
    crontab = crontab_class(context_wrap(CRONTAB_EXTENSIONS))
    assert crontab is not None
    assert crontab.invalid_lines == []
    assert crontab[0] == {
        'time': '@reboot',
        'command': '/usr/local/bin/unmangle_database',
    }
    assert crontab[1] == {
        'minute': '0',
        'hour': '0',
        'day_of_month': '*',
        'month': '*',
        'day_of_week': '*',
        'command': '/usr/sbin/logrotate',
    }


def test_crontab():
    crontab_tests(CrontabL)


def test_heat_crontab():
    crontab_tests(HeatCrontab)


def test_keystone_crontab():
    crontab_tests(KeystoneCrontab)


def test_nova_crontab():
    crontab_tests(NovaCrontab)


def test_root_crontab():
    crontab_tests(RootCrontab)



""""
``test lsblk``
================
"""
from insights.parsers import lsblk, ParseException
from insights.tests import context_wrap

import pytest

LSBLK_DATA = """
NAME          MAJ:MIN RM  SIZE RO TYPE MOUNTPOINT
vda           252:0    0    9G  0 disk
|-vda1        252:1    0  500M  0 part /boot
`-vda2        252:2    0  8.5G  0 part
  |-rhel-root 253:0    0  7.6G  0 lvm  /
  |-rhel-swap 253:1    0  924M  0 lvm  [SWAP]
sda             8:0    0  500G  0 disk
`-sda1          8:1    0  500G  0 part /data
"""

LSBLK_DATA2 = """
NAME                       MAJ:MIN RM    SIZE RO TYPE  MOUNTPOINT
sr0                         11:0    1   64.3M  0 rom
sda                          8:0    0     25G  0 disk
|-sda1                       8:1    0    256M  0 part  /boot
|-sda2                       8:2    0   18.6G  0 part
| `-vg_root-lv_root (dm-0) 253:0    0   18.5G  0 lvm   /
`-sda3                       8:3    0    6.2G  0 part  [SWAP]
sdb                          8:16   0 1000.1G  0 disk
`-mpathb (dm-2)            253:2    0 1000.1G  0 mpath
  `-mpathbp1 (dm-4)        253:4    0 1000.1G  0 part
    `-appdg-app (dm-7)     253:7    0    2.8T  0 lvm   /splunk
sdc                          8:32   0  850.1G  0 disk
`-mpatha (dm-1)            253:1    0  850.1G  0 mpath
  `-mpathap1 (dm-5)        253:5    0  850.1G  0 part
    `-appdg-app (dm-7)     253:7    0    2.8T  0 lvm   /splunk
sdd                          8:48   0 1000.1G  0 disk
`-mpathc (dm-3)            253:3    0 1000.1G  0 mpath
  `-mpathcp1 (dm-6)        253:6    0 1000.1G  0 part
    `-appdg-app (dm-7)     253:7    0    2.8T  0 lvm   /splunk
sde                          8:64   0 1000.1G  0 disk
`-mpathb (dm-2)            253:2    0 1000.1G  0 mpath
  `-mpathbp1 (dm-4)        253:4    0 1000.1G  0 part
    `-appdg-app (dm-7)     253:7    0    2.8T  0 lvm   /splunk
sdf                          8:80   0  850.1G  0 disk
`-mpatha (dm-1)            253:1    0  850.1G  0 mpath
  `-mpathap1 (dm-5)        253:5    0  850.1G  0 part
    `-appdg-app (dm-7)     253:7    0    2.8T  0 lvm   /splunk
sdg                          8:96   0 1000.1G  0 disk
`-mpathc (dm-3)            253:3    0 1000.1G  0 mpath
  `-mpathcp1 (dm-6)        253:6    0 1000.1G  0 part
    `-appdg-app (dm-7)     253:7    0    2.8T  0 lvm   /splunk
"""

LSBLK_DATA3 = """
NAME                 MAJ:MIN RM  SIZE RO TYPE  MOUNTPOINT
vda                  252:0    0   10G  0 disk
|-vda1               252:1    0  501M  0 part
| `-md127              9:127  0  500M  0 raid1 /boot
`-vda2               252:2    0  9.5G  0 part
  `-md126              9:126  0  9.5G  0 raid1
    |-rhel-root 253:0    0  9.4G  0 lvm   /
    `-rhel-swap 253:1    0  100M  0 lvm   [SWAP]
vdb                  252:16   0   10G  0 disk
|-vdb1               252:17   0  501M  0 part
| `-md127              9:127  0  500M  0 raid1 /boot
`-vdb2               252:18   0  9.5G  0 part
  `-md126              9:126  0  9.5G  0 raid1
    |-rhel-root 253:0    0  9.4G  0 lvm   /
    `-rhel-swap 253:1    0  100M  0 lvm   [SWAP]
"""

# lsblk -P -o
LSBLK_EXT_DATA = """
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="" GROUP="cdrom" KNAME="sr0" LABEL="" LOG-SEC="512" MAJ:MIN="11:0" MIN-IO="512" MODE="brw-rw----" MODEL="DVD+-RW DVD8801 " MOUNTPOINT="" NAME="sr0" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" RM="1" RO="0" ROTA="1" RQ-SIZE="128" SCHED="cfq" SIZE="1024M" STATE="running" TYPE="rom" UUID=""
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="" GROUP="disk" KNAME="sda" LABEL="" LOG-SEC="512" MAJ:MIN="8:0" MIN-IO="512" MODE="brw-rw----" MODEL="WDC WD1600JS-75N" MOUNTPOINT="" NAME="sda" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" RM="0" RO="0" ROTA="1" RQ-SIZE="128" SCHED="cfq" SIZE="149G" STATE="running" TYPE="disk" UUID=""
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="ext4" GROUP="disk" KNAME="sda1" LABEL="" LOG-SEC="512" MAJ:MIN="8:1" MIN-IO="512" MODE="brw-rw----" MODEL="" MOUNTPOINT="/boot" NAME="sda1" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" RM="0" RO="0" ROTA="1" RQ-SIZE="128" SCHED="cfq" SIZE="500M" STATE="" TYPE="part" UUID="c7c4c016-8b00-4ded-bffb-5cc4719b7d45"
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="LVM2_member" GROUP="disk" KNAME="sda2" LABEL="" LOG-SEC="512" MAJ:MIN="8:2" MIN-IO="512" MODE="brw-rw----" MODEL="" MOUNTPOINT="" NAME="sda2" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" RM="0" RO="0" ROTA="1" RQ-SIZE="128" SCHED="cfq" SIZE="148.5G" STATE="" TYPE="part" UUID="fFE3aA-ifqV-09uh-1u18-b3mV-73gK-FApXf1"
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="ext4" GROUP="disk" KNAME="dm-0" LABEL="" LOG-SEC="512" MAJ:MIN="253:0" MIN-IO="512" MODE="brw-rw----" MODEL="" MOUNTPOINT="/" NAME="vg_trex-lv_root" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" RM="0" RO="0" ROTA="1" RQ-SIZE="128" SCHED="" SIZE="50G" STATE="running" TYPE="lvm" UUID="0618daba-8dc0-4a1c-926b-4a0f968da62e"
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="swap" GROUP="disk" KNAME="dm-1" LABEL="" LOG-SEC="512" MAJ:MIN="253:1" MIN-IO="512" MODE="brw-rw----" MODEL="" MOUNTPOINT="[SWAP]" NAME="vg_trex-lv_swap" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" RM="0" RO="0" ROTA="1" RQ-SIZE="128" SCHED="" SIZE="3.4G" STATE="running" TYPE="lvm" UUID="102e1d8a-39c9-4065-ae16-d9cbd7162691"
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="ext4" GROUP="disk" KNAME="dm-2" LABEL="" LOG-SEC="512" MAJ:MIN="253:2" MIN-IO="512" MODE="brw-rw----" MODEL="" MOUNTPOINT="/home" NAME="vg_trex-lv_home" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" RM="0" RO="0" ROTA="1" RQ-SIZE="128" SCHED="" SIZE="95.1G" STATE="running" TYPE="lvm" UUID="eee3252d-de08-4732-9d55-f2e33f878664"
"""

LSBLK_EXT_DATA_FAILED_PATH = """
lsblk: dm-0: failed to get device path
lsblk: dm-1: failed to get device path
lsblk: dm-0: failed to get device path
lsblk: dm-1: failed to get device path
NAME="sda" KNAME="sda" MAJ:MIN="8:0" FSTYPE="" MOUNTPOINT="" LABEL="" UUID="" RA="4096" RO="0" RM="0" MODEL="PERC H710P      " SIZE="931G" STATE="running" OWNER="" GROUP="" MODE="" ALIGNMENT="0" MIN-IO="512" OPT-IO="0" PHY-SEC="512" LOG-SEC="512" ROTA="1" SCHED="deadline" RQ-SIZE="128" TYPE="disk" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0"
"""

LSBLK_INVALID_OPTION = """
/bin/lsblk: invalid option -- 'P'

Usage:
 lsblk [options] [<device> ...]

Options:
 -a, --all            print all devices
 -b, --bytes          print SIZE in bytes rather than in human readable format

For more information see lsblk(8).
"""


def test_lsblk():
    # Test Block methods
    results = lsblk.LSBlock(context_wrap(LSBLK_DATA))
    assert results is not None
    assert len(results) == 7
    rhel_root = None
    sda = None
    for result in results:
        if result.name == 'rhel-root':
            rhel_root = result
        elif result.name == 'sda':
            sda = result
    assert rhel_root is not None
    assert rhel_root.maj_min == "253:0"
    assert rhel_root.removable is False
    assert rhel_root.size == "7.6G"
    assert rhel_root.read_only is False
    assert rhel_root.type == "lvm"
    assert rhel_root.mountpoint == "/"
    assert rhel_root.parent_names == ["vda", "vda2"]
    assert sda is not None
    assert sda.maj_min == "8:0"
    assert sda.removable is False
    assert sda.size == "500G"
    assert sda.read_only is False
    assert sda.type == "disk"
    assert 'mountpoint' not in sda
    assert 'parent_names' not in sda
    assert hasattr(results, 'device_data')
    assert isinstance(results.device_data, dict)
    assert sorted(results.device_data.keys()) == sorted([
        'vda', 'vda1', 'vda2', 'rhel-root', 'rhel-swap', 'sda', 'sda1'
    ])
    assert results.device_data['sda'] == sda
    assert sda.get('MAJ_MIN') == sda.maj_min
    assert str(rhel_root) == 'lvm:rhel-root(/)'
    assert str(sda) == 'disk:sda'

    # Keyword search tests
    assert results.search(TYPE='disk') == [
        results.rows[0], results.rows[5]
    ]

    results = lsblk.LSBlock(context_wrap(LSBLK_DATA2))
    assert results is not None
    assert len(results) == 30
    sr0 = None
    sdf_appdg = None
    for result in results:
        if result.name == "sr0":
            sr0 = result
        elif result.name == "appdg-app (dm-7)" and result.parent_names[0] == "sdf":
            sdf_appdg = result
    assert sr0 is not None
    assert sr0.name == "sr0"
    assert sr0.maj_min == "11:0"
    assert sr0.removable is True
    assert sr0.size == "64.3M"
    assert sr0.read_only is False
    assert sr0.type == "rom"
    assert sdf_appdg is not None
    assert sdf_appdg.name == "appdg-app (dm-7)"
    assert sdf_appdg.maj_min == "253:7"
    assert sdf_appdg.removable is False
    assert sdf_appdg.size == "2.8T"
    assert sdf_appdg.read_only is False
    assert sdf_appdg.type == "lvm"
    assert sdf_appdg.mountpoint == "/splunk"
    assert sdf_appdg.parent_names == ["sdf", "mpatha (dm-1)", "mpathap1 (dm-5)"]

    # LSBLK_DATA3 - aka MD raid lsblk output
    results = lsblk.LSBlock(context_wrap(LSBLK_DATA3))
    assert results is not None
    assert len(results) == 14

    rhel_root = None
    md127 = None
    for result in results:
        if result.name == 'rhel-root':
            rhel_root = result
        elif result.name == 'md127':
            md127 = result

    assert rhel_root is not None
    assert rhel_root.maj_min == "253:0"
    assert rhel_root.removable is False
    assert rhel_root.size == "9.4G"
    assert rhel_root.read_only is False
    assert rhel_root.type == "lvm"
    assert rhel_root.mountpoint == "/"
    assert rhel_root.parent_names == ["vdb", "vdb2", "md126"]

    assert md127 is not None
    assert md127.maj_min == "9:127"
    assert md127.removable is False
    assert md127.size == "500M"
    assert md127.read_only is False
    assert md127.type == "raid1"
    assert md127.mountpoint == "/boot"
    assert md127.parent_names == ["vdb", "vdb1"]

    assert hasattr(results, 'device_data')
    assert isinstance(results.device_data, dict)
    assert sorted(results.device_data.keys()) == sorted([
        'vda', 'vda1', 'vda2', 'rhel-root', 'rhel-swap', 'vdb', 'vdb1', 'vdb2', 'md126', 'md127'
    ])
    assert results.device_data['md127'] == md127
    assert md127.get('MAJ_MIN') == md127.maj_min
    assert str(rhel_root) == 'lvm:rhel-root(/)'
    assert str(md127) == 'raid1:md127(/boot)'


LSBLK_DATA_BAD = """
NAME          MAJ:MIN RM  SIZE RO TYPE MOUNTPOINT
vda           252:0    0    9G  0
|-vda1        252:1    0  500M  0 part /boot
"""


def test_lsblock_bad_data():
    blocks = lsblk.LSBlock(context_wrap(LSBLK_DATA_BAD))
    assert len(blocks.rows) == 1
    assert blocks.rows[0].name == 'vda1'


def test_lsblk_po():
    results = lsblk.LSBlockPairs(context_wrap(LSBLK_EXT_DATA))
    assert results is not None
    assert len(results) == 7
    sda1 = None
    for result in results:
        if result.name == 'sda1':
            sda1 = result
    assert sda1 is not None
    assert sda1.alignment == "0"
    assert sda1.disc_aln == "0"
    assert sda1.disc_gran == "0B"
    assert sda1.disc_max == "0B"
    assert sda1.disc_zero == "0"
    assert sda1.fstype == "ext4"
    assert sda1.group == "disk"
    assert sda1.kname == "sda1"
    assert 'LABEL' not in sda1
    assert sda1.log_sec == "512"
    assert sda1.maj_min == "8:1"
    assert sda1.min_io == "512"
    assert sda1.mode == "brw-rw----"
    assert 'MODEL' not in sda1
    assert sda1.mountpoint == "/boot"
    assert sda1.name == "sda1"
    assert sda1.opt_io == "0"
    assert sda1.owner == "root"
    assert sda1.phy_sec == "512"
    assert sda1.ra == "128"
    assert sda1.removable is False
    assert sda1.read_only is False
    assert sda1.rota == "1"
    assert sda1.rq_size == "128"
    assert sda1.sched == "cfq"
    assert sda1.size == "500M"
    assert 'STATE' not in sda1
    assert sda1.type == "part"
    assert sda1.uuid == "c7c4c016-8b00-4ded-bffb-5cc4719b7d45"


LSBLOCKPAIRS_NO_TYPE_DATA = """
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="" GROUP="cdrom" KNAME="sr0" LABEL="" LOG-SEC="512" MAJ:MIN="11:0" MIN-IO="512" MODE="brw-rw----" MODEL="DVD+-RW DVD8801 " MOUNTPOINT="" NAME="sr0" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" RM="1" RO="0" ROTA="1" RQ-SIZE="128" SCHED="cfq" SIZE="1024M" STATE="running" UUID=""
"""


def test_lsblockpairs_no_type():
    # LSBlock will always have a type column because of the regular expression
    # match; LSBlockPairs is looser about the data available.
    with pytest.raises(ParseException) as exc:
        blocks = lsblk.LSBlockPairs(context_wrap(LSBLOCKPAIRS_NO_TYPE_DATA))
        assert repr(blocks.rows[0]) is None
    assert 'TYPE not found in LsBlockPairs line' in str(exc)


def test_lsblockpairs_failed():
    blocks = lsblk.LSBlockPairs(context_wrap(LSBLK_EXT_DATA_FAILED_PATH))
    assert "dm-0" in blocks.failed_device_paths
    assert "dm-1" in blocks.failed_device_paths


def test_lsblockpairs_invalid_option():
    with pytest.raises(ParseException) as ex:
        lsblk.LSBlockPairs(context_wrap(LSBLK_INVALID_OPTION))
    assert "/bin/lsblk: invalid option" in str(ex)


LSBLOCKPAIRS_NO_TRANSLATE_DATA = """
ALIGNMENT="0" DISC-ALN="0" DISC-GRAN="0B" DISC-MAX="0B" DISC-ZERO="0" FSTYPE="" GROUP="cdrom" KNAME="sr0" LABEL="" LOG-SEC="512" MAJ:MIN="11:0" MIN-IO="512" MODE="brw-rw----" MODEL="DVD+-RW DVD8801 " MOUNTPOINT="" NAME="sr0" OPT-IO="0" OWNER="root" PHY-SEC="512" RA="128" ROTA="1" RQ-SIZE="128" SCHED="cfq" SIZE="1024M" STATE="running" TYPE="rom" UUID=""
"""


def test_lsblockpairs_no_translate():
    # LSBlockPairs translates 'RM' to 'REMOVABLE' and 'RO' to 'READ_ONLY';
    # normally every block device will have these two attributes.  But if for
    # some reason it doesn't, we want to make sure that the code still works
    # correctly.  This is mainly for code coverage testing.
    blocks = lsblk.LSBlockPairs(context_wrap(LSBLOCKPAIRS_NO_TRANSLATE_DATA))
    assert 'sr0' in blocks.device_data
    assert not hasattr(blocks.device_data['sr0'], 'removable')
    assert not hasattr(blocks.device_data['sr0'], 'read_only')



from insights.parsers import SkipException
from insights.parsers import x86_debug
from insights.parsers.x86_debug import X86IBPBEnabled
from insights.parsers.x86_debug import X86PTIEnabled
from insights.parsers.x86_debug import X86IBRSEnabled
from insights.parsers.x86_debug import X86RETPEnabled
from insights.tests import context_wrap
import pytest
import doctest


def test_x86_ibpb_enabled():
    ibpb = X86IBPBEnabled(context_wrap('0'))
    assert ibpb.value == 0
    assert isinstance(ibpb.value, int)

    ibpb = X86IBPBEnabled(context_wrap('1'))
    assert ibpb.value == 1
    assert isinstance(ibpb.value, int)

    ibpb = X86IBPBEnabled(context_wrap('2'))
    assert ibpb.value == 2
    assert isinstance(ibpb.value, int)


def test_x86_pti_enabled():
    pti = X86PTIEnabled(context_wrap('0'))
    assert pti.value == 0
    assert isinstance(pti.value, int)

    pti = X86PTIEnabled(context_wrap('1'))
    assert pti.value == 1
    assert isinstance(pti.value, int)


def test_x86_ibrs_enabled():
    ibrs = X86IBRSEnabled(context_wrap('0'))
    assert ibrs.value == 0
    assert isinstance(ibrs.value, int)

    ibrs = X86IBRSEnabled(context_wrap('1'))
    assert ibrs.value == 1
    assert isinstance(ibrs.value, int)

    ibrs = X86IBRSEnabled(context_wrap('2'))
    assert ibrs.value == 2
    assert isinstance(ibrs.value, int)


def test_x86_retp_enabled():
    retp = X86RETPEnabled(context_wrap('0'))
    assert retp.value == 0
    assert isinstance(retp.value, int)

    retp = X86RETPEnabled(context_wrap('1'))
    assert retp.value == 1
    assert isinstance(retp.value, int)

    retp = X86RETPEnabled(context_wrap('2'))
    assert retp.value == 2
    assert isinstance(retp.value, int)


def test_x86_enabled_documentation():
    """
    Here we test the examples in the documentation automatically using
    doctest.  We set up an environment which is similar to what a rule
    writer might see - a '/sys/kernel/debug/x86/*_enabled' output
    that has been passed in as a parameter to the rule declaration.
    """
    env = {
        'dva': X86IBPBEnabled(context_wrap('1')),
        'dv': X86PTIEnabled(context_wrap('1')),
        'dl': X86IBRSEnabled(context_wrap('1')),
        'dval': X86RETPEnabled(context_wrap('1'))}

    failed, total = doctest.testmod(x86_debug, globs=env)
    assert failed == 0


def test_x86_ibpb_enabled_exp():
    """
    Here test the examples cause expections
    """
    # ibpb
    with pytest.raises(SkipException) as sc1:
        X86IBPBEnabled(context_wrap(""))
    assert "Input content is empty" in str(sc1)


def test_x86_pti_enabled_exp():
    """
    Here test the examples cause expections
    """
    # pti
    with pytest.raises(SkipException) as sc2:
        X86PTIEnabled(context_wrap(""))
    assert "Input content is empty" in str(sc2)


def test_x86_ibrs_enabled_exp():
    """
    Here test the examples cause expections
    """
    # ibrs
    with pytest.raises(SkipException) as sc3:
        X86IBRSEnabled(context_wrap(""))
    assert "Input content is empty" in str(sc3)


def test_x86_retp_enabled_exp():
    """
    Here test the examples cause expections
    """
    # retp
    with pytest.raises(SkipException) as sc4:
        X86RETPEnabled(context_wrap(""))
    assert "Input content is empty" in str(sc4)



import pytest
import doctest

from insights import SkipComponent
from insights.parsers import yum_list
from insights.parsers.yum_list import YumListInstalled
from insights.tests import context_wrap


EMPTY = """
Installed Packages
""".strip()

EXPIRED_EMPTY = """
Repodata is over 2 weeks old. Install yum-cron? Or run: yum makecache fast
Installed Packages
""".strip()

EXPIRED_WITH_DATA = """
Repodata is over 2 weeks old. Install yum-cron? Or run: yum makecache fast
Installed Packages
bash.x86_64                               4.4.23-1.fc28                 @updates
""".strip()

SIMPLE = """
Installed Packages
bash.x86_64                               4.4.23-1.fc28                 @updates
""".strip()

WRAPPED_LINE = """
Installed Packages
NetworkManager-bluetooth.x86_64           1:1.10.10-1.fc28              @updates
NetworkManager-config-connectivity-fedora.noarch
                                          1:1.10.10-1.fc28              @updates
NetworkManager-glib.x86_64                1:1.10.10-1.fc28              @updates
NetworkManager-libnm.x86_64               1:1.10.10-1.fc28              @updates
clucene-contribs-lib.x86_64               2.3.3.4-31.20130812.e8e3d20git.fc28
                                                                        @fedora
clucene-core.x86_64                       2.3.3.4-31.20130812.e8e3d20git.fc28
                                                                        @fedora
""".strip()

COMMANDLINE = """
Installed Packages
jdk1.8.0_121.x86_64                       2000:1.8.0_121-fcs            @@commandline
"""

HEADER_FOOTER_JUNK = """
Loaded plugins: product-id, search-disabled-repos, subscription-manager
Repodata is over 2 weeks old. Install yum-cron? Or run: yum makecache fast
Installed Packages
GConf2.x86_64                    3.2.6-8.el7             @rhel-7-server-rpms
GeoIP.x86_64                     1.5.0-11.el7            @anaconda/7.3
ImageMagick.x86_64               6.7.8.9-15.el7_2        @rhel-7-server-rpms
NetworkManager.x86_64            1:1.4.0-17.el7_3        installed
NetworkManager.x86_64            1:1.8.0-9.el7           installed
NetworkManager-config-server.noarch
                                 1:1.8.0-9.el7           installed
Uploading Enabled Repositories Report
Loaded plugins: priorities, product-id, rhnplugin, rhui-lb, subscription-
              : manager, versionlock
"""


def test_empty():
    ctx = context_wrap(EMPTY)
    with pytest.raises(SkipComponent):
        YumListInstalled(ctx)


def test_simple():
    ctx = context_wrap(SIMPLE)
    rpms = YumListInstalled(ctx)
    rpm = rpms.newest("bash")
    assert rpm is not None
    assert rpm.epoch == "0"
    assert rpm.version == "4.4.23"
    assert rpm.release == "1.fc28"
    assert rpm.arch == "x86_64"
    assert rpm.repo == "updates"


def test_expired_cache_with_data():
    ctx = context_wrap(EXPIRED_WITH_DATA)
    rpms = YumListInstalled(ctx)
    assert rpms.expired_cache is True


def test_expired_cache_no_data():
    ctx = context_wrap(EXPIRED_EMPTY)
    with pytest.raises(SkipComponent):
        YumListInstalled(ctx)


def test_wrapped():
    ctx = context_wrap(WRAPPED_LINE)
    rpms = YumListInstalled(ctx)
    rpm = rpms.newest("NetworkManager-bluetooth")
    assert rpm is not None
    assert rpm.epoch == "1"
    assert rpm.version == "1.10.10"
    assert rpm.release == "1.fc28"
    assert rpm.arch == "x86_64"
    assert rpm.repo == "updates"

    rpm = rpms.newest("NetworkManager-config-connectivity-fedora")
    assert rpm is not None
    assert rpm.epoch == "1"
    assert rpm.version == "1.10.10"
    assert rpm.release == "1.fc28"
    assert rpm.arch == "noarch"
    assert rpm.repo == "updates"

    rpm = rpms.newest("clucene-contribs-lib")
    assert rpm is not None
    assert rpm.epoch == "0"
    assert rpm.version == "2.3.3.4"
    assert rpm.release == "31.20130812.e8e3d20git.fc28"
    assert rpm.arch == "x86_64"
    assert rpm.repo == "fedora"

    rpm = rpms.newest("clucene-core")
    assert rpm is not None
    assert rpm.epoch == "0"
    assert rpm.version == "2.3.3.4"
    assert rpm.release == "31.20130812.e8e3d20git.fc28"
    assert rpm.arch == "x86_64"
    assert rpm.repo == "fedora"


def test_commandline():
    ctx = context_wrap(COMMANDLINE)
    rpms = YumListInstalled(ctx)

    rpm = rpms.newest("jdk1.8.0_121")
    assert rpm is not None
    assert rpm.epoch == "2000"
    assert rpm.version == "1.8.0_121"
    assert rpm.release == "fcs"
    assert rpm.arch == "x86_64"
    assert rpm.repo == "commandline"


def test_multiple_stanza():
    ctx = context_wrap(HEADER_FOOTER_JUNK)
    rpms = YumListInstalled(ctx)

    rpm = rpms.newest("GConf2")
    assert rpm is not None
    assert rpm.epoch == "0"
    assert rpm.version == "3.2.6"
    assert rpm.release == "8.el7"
    assert rpm.arch == "x86_64"
    assert rpm.repo == "rhel-7-server-rpms"

    rpm = rpms.newest("NetworkManager-config-server")
    assert rpm is not None
    assert rpm.epoch == "1"
    assert rpm.version == "1.8.0"
    assert rpm.release == "9.el7"
    assert rpm.arch == "noarch"
    assert rpm.repo == "installed"


def test_doc_examples():
    env = {
        'installed_rpms': YumListInstalled(context_wrap(HEADER_FOOTER_JUNK)),
    }
    failed, total = doctest.testmod(yum_list, globs=env)
    assert failed == 0



"""
DockerInspect - Command ``docker inspect --type={TYPE}``
========================================================

This module parses the output of the ``docker inspect`` command.  This uses
the ``core.marshalling.unmarshal`` function to parse the JSON output from the
commands.  The parsed data can be accessed a dictionary via the object itself.

"""

from insights import parser, CommandParser
from insights.core.marshalling import unmarshal
from insights.parsers import SkipException
from insights.specs import Specs
from insights.util import deprecated


class DockerInspect(CommandParser, dict):
    """
    .. warning::
        This class is deprecated, please use
        :py:class:`insights.parsers.containers_inspect.ContainersInspect` instead.

    Parse the output of command "docker inspect --type=image" and "docker
    inspect --type=container".  The output of these two commands is formatted
    as JSON, so "json.loads" is an option to parse the output in the future.

    Raises:
        SkipException: If content is not provided
    """
    def __init__(self, *args, **kwargs):
        deprecated(
            DockerInspect,
            "Please use the :class:`insights.parsers.containers_inspect.ContainersInspect` instead.",
            "3.2.25"
        )
        super(DockerInspect, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        if not content:
            raise SkipException

        content = "\n".join(list(content))
        try:
            inspect_data = unmarshal(content)
            self.update(inspect_data[0])
        except:
            raise SkipException

    @property
    def data(self):
        return self


@parser(Specs.docker_image_inspect)
class DockerInspectImage(DockerInspect):
    """
    Parse docker image inspect output using the DockerInspect parser class.

    Sample input::

        [
            {
                "Id": "882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2",
                "RepoTags": [
                    "rhel7_imagemagick:latest"
                ],
                "RepoDigests": [],
        ...

    Examples:

        >>> image['Id'] == '882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2'
        True
        >>> image['RepoTags'][0] == 'rhel7_imagemagick:latest'
        True

    """
    pass


@parser(Specs.docker_container_inspect)
class DockerInspectContainer(DockerInspect):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.containers_inspect.ContainersInspect` instead.

    Parse docker container inspect output using the DockerInspect parser class.

    Sample input::

        [
            {
                "Id": "97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07",
                "Created": "2016-06-23T05:12:25.433469799Z",
                "Path": "/bin/bash",
                "Args": [],
                "Name": "/hehe2",
                "State": {
                    "Status": "running",
                    "Running": true,
                    "Paused": false,
                    "Restarting": false,
            ...

    Examples:

        >>> container['Id'] == '97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07'
        True
        >>> container['Name'] == '/hehe2'
        True
        >>> container.get('State').get('Paused') # sub-dictionaries
        False

    """
    def __init__(self, *args, **kwargs):
        deprecated(
            DockerInspectContainer,
            "Please use the :class:`insights.parsers.containers_inspect.ContainersInspect` instead.",
            "3.2.25"
        )
        super(DockerInspectContainer, self).__init__(*args, **kwargs)



"""
Hostname - command ``hostname``
===============================

Parsers contained in this module are:

Hostname - command ``hostname -f``
----------------------------------

HostnameDefault - command ``hostname``
--------------------------------------

HostnameShort - command ``hostname -s``
---------------------------------------
"""

from insights.parsers import ParseException
from insights import parser, CommandParser
from insights.specs import Specs


class HostnameBase(CommandParser):
    """
    The base parser class for command ``hostname``.

    Raises:
        ParseException: When the output contains multiple non-empty lines.
    """
    def parse_content(self, content):
        content = list(filter(None, content))
        if len(content) != 1:
            msg = "hostname data contains multiple non-empty lines"
            raise ParseException(msg)
        self.raw = content[0].strip()
        self.hostname = self.raw.split(".")[0] if self.raw else None


@parser(Specs.hostname)
class Hostname(HostnameBase):
    """
    This parser simply reads the output of ``hostname -f``, which is the
    configured fully qualified domain name of the client system.  It then
    splits it into ``hostname`` and ``domain`` and stores these as attributes,
    along with the unmodified name in the ``fqdn`` attribute.

    Examples:
        >>> hostname.raw
        'rhel7.example.com'
        >>> hostname.fqdn
        'rhel7.example.com'
        >>> hostname.hostname
        'rhel7'
        >>> hostname.domain
        'example.com'

    Attributes:
        raw: The raw output of the ``hostname -f`` command.
        fqdn: The fully qualified domain name of the host. The same to
            ``hostname`` when domain part is not set.
        hostname: The hostname.
        domain: The domain get from the ``fqdn``.
    """
    def parse_content(self, content):
        super(Hostname, self).parse_content(content)
        self.fqdn = self.raw
        self.domain = ".".join(self.raw.split(".")[1:]) if self.raw else None

    def __repr__(self):
        return "<raw: {r}, hostname: {h}, domain: {d}>".format(r=self.raw, h=self.hostname, d=self.domain)


@parser(Specs.hostname_default)
class HostnameDefault(HostnameBase):
    """
    This parser simply reads the output of ``hostname``.

    Examples:
        >>> hostname_def.raw
        'rhel7'
        >>> hostname_def.hostname
        'rhel7'

    Attributes:
        raw: The raw output of the ``hostname`` command.
        hostname: The hostname.
    """
    def __repr__(self):
        return "<raw: {r}, hostname: {h}>".format(r=self.raw, h=self.hostname)


@parser(Specs.hostname_short)
class HostnameShort(HostnameBase):
    """
    This parser simply reads the output of ``hostname -s``, which is the
    configured short hostname of the client system.

    Examples:
        >>> hostname_s.raw
        'rhel7'
        >>> hostname_s.hostname
        'rhel7'

    Attributes:
        raw: The raw output of the ``hostname -s`` command.
        hostname: The hostname.
    """
    def __repr__(self):
        return "<raw: {r}, hostname: {h}>".format(r=self.raw, h=self.hostname)



import doctest

from insights.parsers import ls_osroot
from insights.parsers.ls_osroot import LsOsroot
from insights.tests import context_wrap

LS_LAN_OSROOT = """
total 5256
dr-xr-xr-x.  17 0 0     271 Apr  5 18:08 .
dr-xr-xr-x.  17 0 0     271 Apr  5 18:08 ..
-rw-r--r--.   1 0 0       0 Feb 25  2017 1
lrwxrwxrwx.   1 0 0       7 Feb 25  2017 bin -> usr/bin
dr-xr-xr-x.   3 0 0    4096 Feb 25  2017 boot
-rw-r--r--.   1 0 0 5168141 Oct 16  2017 channel-list
drwxr-xr-x.  21 0 0    3440 Apr 12 14:46 dev
drwxr-xr-x. 148 0 0    8192 Apr 18 09:17 etc
drwxr-xr-x.   5 0 0      37 Jul 31  2017 home
lrwxrwxrwx.   1 0 0       7 Feb 25  2017 lib -> usr/lib
lrwxrwxrwx.   1 0 0       9 Feb 25  2017 lib64 -> usr/lib64
drwxr-xr-x.   2 0 0       6 Mar 10  2016 media
drwxr-xr-x.   2 0 0       6 Mar 10  2016 mnt
drwxr-xr-x.   5 0 0      48 Mar 27 13:37 opt
dr-xr-xr-x. 265 0 0       0 Apr  6 02:07 proc
-rw-r--r--.   1 0 0  175603 Apr  5 18:08 .readahead
dr-xr-x---.  26 0 0    4096 Apr 18 09:17 root
drwxr-xr-x.  43 0 0    1340 Apr 18 09:17 run
lrwxrwxrwx.   1 0 0       8 Feb 25  2017 sbin -> usr/sbin
drwxr-xr-x.   2 0 0       6 Mar 10  2016 srv
dr-xr-xr-x.  13 0 0       0 Apr  5 18:07 sys
drwxrwxrwt.  40 0 0    8192 Apr 18 11:17 tmp
drwxr-xr-x.  13 0 0     155 Feb 25  2017 usr
drwxr-xr-x.  21 0 0    4096 Apr  6 02:07 var
""".strip()


def test_ls_osroot():
    ls_osroot = LsOsroot(context_wrap(LS_LAN_OSROOT, path='ls_-lan'))
    assert '/' in ls_osroot
    assert len(ls_osroot.files_of("/")) == 7
    assert ls_osroot.files_of("/") == ['1', 'bin', 'channel-list', 'lib', 'lib64', '.readahead', 'sbin']
    assert ls_osroot.dirs_of("/") == ['.', '..', 'boot', 'dev', 'etc', 'home', 'media', 'mnt', 'opt', 'proc', 'root', 'run', 'srv', 'sys', 'tmp', 'usr', 'var']
    assert ls_osroot.listing_of("/")['tmp'] == {'group': '0', 'name': 'tmp', 'links': 40, 'perms': 'rwxrwxrwt.', 'raw_entry': 'drwxrwxrwt.  40 0 0    8192 Apr 18 11:17 tmp', 'owner': '0', 'date': 'Apr 18 11:17', 'type': 'd', 'dir': '/', 'size': 8192}


def test_ls_osroot_doc_examples():
    env = {
        'LsOsroot': LsOsroot,
        'ls_osroot': LsOsroot(context_wrap(LS_LAN_OSROOT, path='ls_-lan')),
    }
    failed, total = doctest.testmod(ls_osroot, globs=env)
    assert failed == 0



"""
AMQBroker - file ``/var/opt/amq-broker/*/etc/broker.xml``
=========================================================

Configuration of Active MQ Artemis brokers.
"""
from .. import XMLParser, parser
from insights.specs import Specs


@parser(Specs.amq_broker)
class AMQBroker(XMLParser):
    """
    Provides access to broker.xml files that are stored in the conventional
    location for Active MQ Artemis.

    Examples:
        >>> doc.get_elements(".//journal-pool-files", "urn:activemq:core")[0].text
        "10"
        >>> doc.get_elements(".//journal-type", "urn:activemq:core")[0].text
        "NIO"
    """
    pass



"""
LsVarLibMongodb - command ``ls -la /var/lib/mongodb``
=====================================================

The ``ls -la /var/lib/mongodb`` command provides information for the listing of the ``/var/lib/mongodb`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Sample directory list::

    total 6322200
    drwxr-xr-x.  3 mongodb mongodb        256 Jun  7 10:07 .
    drwxr-xr-x. 71 root    root          4096 Jun 22 10:35 ..
    drwxr-xr-x.  2 mongodb mongodb         65 Jul 10 09:33 journal
    -rw-------.  1 mongodb mongodb   67108864 Jul 10 09:32 local.0
    -rw-------.  1 mongodb mongodb   16777216 Jul 10 09:32 local.ns

Examples:

    >>> "journal" in ls_var_lib_mongodb
    False
    >>> "/var/lib/mongodb" in ls_var_lib_mongodb
    True
    >>> ls_var_lib_mongodb.dir_entry('/var/lib/mongodb', 'journal')['type']
    'd'
"""


from insights.specs import Specs

from .. import CommandParser, parser
from .. import FileListing


@parser(Specs.ls_var_lib_mongodb)
class LsVarLibMongodb(CommandParser, FileListing):
    """Parses output of ``ls -la /var/lib/mongodb`` command."""
    pass



"""
CephLog - file ``/var/log/ceph/ceph.log``
=========================================
"""

from insights import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.ceph_log)
class CephLog(LogFileOutput):
    """
      Parse the ``/var/log/ceph/ceph.log`` file.

      Provide access to ceph logs using the LogFileOutput parser class.

      Sample log lines::

          2017-05-31 13:01:44.034376 mon.0 192.xx.xx.xx:6789/0 742585 : cluster [INF] pgmap v5133969: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 2027 kB/s rd, 20215 kB/s wr, 711 op/s
          2017-05-31 13:01:45.041760 mon.0 192.xx.xx.xx:6789/0 742586 : cluster [INF] pgmap v5133970: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 1606 kB/s rd, 17354 kB/s wr, 718 op/s
          2017-05-31 13:01:46.933829 osd.22 192.xx.xx.xx:6814/42154 172581 : cluster [WRN] 44 slow requests, 2 included below; oldest blocked for > 49.982746 secs
          2017-05-31 13:01:46.933946 osd.22 192.xx.xx.xx:6814/42154 172582 : cluster [WRN] slow request 30.602517 seconds old, received at 2017-05-31 13:01:06.330484: osd_op(client.3395798.0:2855671 1.54392173 gnocchi_06c8214c-afae-4e64-8a4a-a466c4f257dc_1244160000.0_median_86400.0_v3 [write 26253~9] snapc 0=[] ondisk+write+known_if_redirected e487) currently waiting for subops from 23
          2017-05-31 13:01:46.933955 osd.22 192.xx.xx.xx:6814/42154 172583 : cluster [WRN] slow request 30.530961 seconds old, received at 2017-05-31 13:01:06.402041: osd_op(client.324182.0:46141816 1.e637a4b3 measure [omap-rm-keys 0~107] snapc 0=[] ondisk+write+skiprwlocks+known_if_redirected e487) currently waiting for subops from 23
          2017-05-31 13:01:47.050539 mon.0 192.xx.xx.xx:6789/0 742589 : cluster [INF] pgmap v5133971: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 1597 kB/s rd, 7259 kB/s wr, 398 op/s
          2017-05-31 13:01:48.057187 mon.0 192.xx.xx.xx:6789/0 742590 : cluster [INF] pgmap v5133972: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 2373 kB/s rd, 5138 kB/s wr, 354 op/s
          2017-05-31 13:01:49.064950 mon.0 192.xx.xx.xx:6789/0 742598 : cluster [INF] pgmap v5133973: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 4187 kB/s rd, 10266 kB/s wr, 714 op/s
          2017-05-31 13:01:50.069437 mon.0 192.xx.xx.xx:6789/0 742599 : cluster [INF] pgmap v5133974: 320 pgs: 3 active+clean+scrubbing+deep, 317 active+clean; 898 GB data, 1828 GB used, 48447 GB / 50275 GB avail; 470 MB/s rd, 11461 kB/s wr, 786 op/s

      Examples:
          >>> len(ceph_log.get("[WRN] slow request")) == 2
          True
          >>> from datetime import datetime
          >>> len(list(ceph_log.get_after(datetime(2017, 5, 31, 13, 1, 46))))
          7

      """
    pass



import logging
import six
import sys

from collections import defaultdict
from datetime import datetime

from ..formats import Formatter
from ..specs import Specs
from ..combiners.hostname import Hostname as combiner_hostname
from ..parsers.branch_info import BranchInfo
from ..util import utc
from . import dr, plugins
from .context import ExecutionContext
import insights

log = logging.getLogger(__name__)


def get_simple_module_name(obj):
    return dr.BASE_MODULE_NAMES.get(obj, None)


class Evaluator(Formatter):
    def __init__(self, broker=None, stream=sys.stdout, incremental=False):
        super(Evaluator, self).__init__(broker or dr.Broker(), stream)
        self.results = defaultdict(list)
        self.rule_skips = []
        self.hostname = None
        self.metadata = {}
        self.metadata_keys = {}
        self.incremental = incremental
        self.context_cls = None

    def observer(self, comp, broker):
        if self.context_cls is None:
            for c in self.broker.instances:
                try:
                    if issubclass(c, ExecutionContext):
                        self.context_cls = c
                except:
                    pass

        if comp is combiner_hostname and comp in broker:
            self.hostname = broker[comp].fqdn

        if plugins.is_rule(comp) and comp in broker:
            self.handle_result(comp, broker[comp])

    def preprocess(self):
        self.broker.add_observer(self.observer)

    def run_serial(self, graph=None):
        dr.run(graph or dr.COMPONENTS[dr.GROUPS.single], broker=self.broker)

    def run_incremental(self, graph=None, parallel=False):
        components = graph or dr.COMPONENTS[dr.GROUPS.single]
        if parallel:
            with insights.get_pool(parallel, "insights-engine-pool", {"max_workers": None}) as pool:
                dr.run_all(components, self.broker, pool)
        else:
            dr.run_all(components, self.broker)

    def format_response(self, response):
        """
        To be overridden by subclasses to format the response sent back to the
        client.
        """
        return response

    def format_result(self, result):
        """
        To be overridden by subclasses to format individual rule results.
        """
        return result

    def process(self, graph=None, parallel=False):
        with self:
            if self.incremental:
                self.run_incremental(graph, parallel)
            else:
                self.run_serial(graph)
        return self.get_response()


class SingleEvaluator(Evaluator):
    def append_metadata(self, r):
        for k, v in r.items():
            if k != "type":
                self.metadata[k] = v

    def format_response(self, response):
        return response

    def get_response(self):
        r = dict(self.metadata_keys)
        r.update({
            "system": {
                "metadata": self.metadata,
                "hostname": self.hostname
            },
            "reports": self.results["rule"],
            "fingerprints": self.results["fingerprint"],
            "skips": self.rule_skips,
        })

        for k, v in six.iteritems(self.results):
            if k not in ("rule", "fingerprint"):
                r[k] = v

        r = self.format_response(r)

        ctx = dr.get_name(self.context_cls) if self.context_cls is not None else None

        r["analysis_metadata"] = {
            "start": self.start_time.isoformat(),
            "finish": datetime.now(utc).isoformat(),
            "execution_context": ctx,
            "plugin_sets": insights.RULES_STATUS
        }

        return r

    def handle_result(self, plugin, r):
        type_ = r["type"]

        if type_ == "skip":
            self.rule_skips.append(r)
        elif type_ == "metadata":
            self.append_metadata(r)
        elif type_ == "metadata_key":
            self.metadata_keys[r.get_key()] = r["value"]
        else:
            response_id = "%s_id" % r.response_type
            key = r.get_key()
            self.results[type_].append(self.format_result({
                response_id: "{0}|{1}".format(get_simple_module_name(plugin), key),
                "component": dr.get_name(plugin),
                "type": type_,
                "key": key,
                "details": r,
                "tags": list(dr.get_tags(plugin)),
                "links": dr.get_delegate(plugin).links or {}
            }))


class InsightsEvaluator(SingleEvaluator):
    def __init__(self, broker=None, system_id=None, stream=sys.stdout, incremental=False):
        super(InsightsEvaluator, self).__init__(broker, stream=stream, incremental=incremental)
        self.system_id = system_id
        self.branch_info = {}
        self.product = "rhel"
        self.type = "host"
        self.release = None

    def observer(self, comp, broker):
        super(InsightsEvaluator, self).observer(comp, broker)
        if self.system_id is None and Specs.machine_id in broker:
            self.system_id = broker[Specs.machine_id].content[0].strip()

        if self.release is None and Specs.redhat_release in broker:
            self.release = broker[Specs.redhat_release].content[0].strip()

        if not self.branch_info and BranchInfo in broker:
            self.branch_info = broker[BranchInfo].data

        if comp is Specs.metadata_json and comp in broker:
            md = broker[comp]
            self.product = md.get("product_code")
            self.type = md.get("role")

    def format_result(self, result):
        result["system_id"] = self.system_id
        return result

    def format_response(self, response):
        system = response["system"]
        system["remote_branch"] = self.branch_info.get("remote_branch")
        system["remote_leaf"] = self.branch_info.get("remote_leaf")
        system["system_id"] = self.system_id
        system["product"] = self.product
        system["type"] = self.type
        if self.release:
            system["metadata"]["release"] = self.release

        return response



import pytest
from mock.mock import Mock
from insights.core.dr import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.sys_fs_cgroup_memory_tasks_number import sys_fs_cgroup_memory_tasks_number_data_datasource, LocalSpecs


SYS_FS_CGROUP_MEMORY_TASKS = """
/sys/fs/cgroup/memory/user.slice/tasks
/sys/fs/cgroup/memory/system.slice/rh-nginx120-nginx.service/tasks
/sys/fs/cgroup/memory/system.slice/named.service/tasks
/sys/fs/cgroup/memory/system.slice/rhel-push-plugin.service/tasks
/sys/fs/cgroup/memory/system.slice/docker.service/tasks
/sys/fs/cgroup/memory/system.slice/var-lib-docker-overlay2.mount/tasks
/sys/fs/cgroup/memory/system.slice/var-lib-docker-containers.mount/tasks
/sys/fs/cgroup/memory/system.slice/sys-kernel-debug.mount/tasks
/sys/fs/cgroup/memory/system.slice/systemd-update-utmp.service/tasks
/sys/fs/cgroup/memory/system.slice/sshd.service/tasks
/sys/fs/cgroup/memory/system.slice/rhsmcertd.service/tasks
""".strip()

SYS_FS_CGROUP_MEMORY_TASKS_NO_FILE = """"""

NG_COMMAND_1 = """
/usr/bin/find: '/sys/fs/cgroup/memory': No such file or directory
""".strip()

NG_COMMAND_2 = """
-bash: /usr/bin/find: No such file or directory
""".strip()

SYS_FS_CGROUP_MEMORY_TASKS_RESULT = """
11
""".strip()

SYS_FS_CGROUP_MEMORY_TASKS_RESULT_NO_FILE = """
0
""".strip()

RELATIVE_PATH = 'insights_commands/sys_fs_cgroup_memory_tasks_number'


def test_sys_fs_cgroup_memory_tasks_number_data_datasource():
    sys_fs_cgroup_memory_tasks_number_data = Mock()
    sys_fs_cgroup_memory_tasks_number_data.content = SYS_FS_CGROUP_MEMORY_TASKS.splitlines()
    broker = {LocalSpecs.sys_fs_cgroup_memory_tasks_raw: sys_fs_cgroup_memory_tasks_number_data}
    result = sys_fs_cgroup_memory_tasks_number_data_datasource(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=SYS_FS_CGROUP_MEMORY_TASKS_RESULT, relative_path=RELATIVE_PATH)
    assert result.content == expected.content
    assert result.relative_path == expected.relative_path


def test_sys_fs_cgroup_memory_tasks_number_data_datasource_no_file():
    sys_fs_cgroup_memory_tasks_number_data = Mock()
    sys_fs_cgroup_memory_tasks_number_data.content = SYS_FS_CGROUP_MEMORY_TASKS_NO_FILE.splitlines()
    broker = {LocalSpecs.sys_fs_cgroup_memory_tasks_raw: sys_fs_cgroup_memory_tasks_number_data}
    result = sys_fs_cgroup_memory_tasks_number_data_datasource(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=SYS_FS_CGROUP_MEMORY_TASKS_RESULT_NO_FILE, relative_path=RELATIVE_PATH)
    assert result.content == expected.content
    assert result.relative_path == expected.relative_path


def test_sys_fs_cgroup_memory_tasks_number_data_datasource_NG_1_output():
    sys_fs_cgroup_memory_tasks_number_data = Mock()
    sys_fs_cgroup_memory_tasks_number_data.content = NG_COMMAND_1.splitlines()
    broker = {LocalSpecs.sys_fs_cgroup_memory_tasks_raw: sys_fs_cgroup_memory_tasks_number_data}
    with pytest.raises(SkipComponent) as e:
        sys_fs_cgroup_memory_tasks_number_data_datasource(broker)
    assert 'SkipComponent' in str(e)


def test_sys_fs_cgroup_memory_tasks_number_data_datasource_NG_2_output():
    sys_fs_cgroup_memory_tasks_number_data = Mock()
    sys_fs_cgroup_memory_tasks_number_data.content = NG_COMMAND_2.splitlines()
    broker = {LocalSpecs.sys_fs_cgroup_memory_tasks_raw: sys_fs_cgroup_memory_tasks_number_data}
    with pytest.raises(SkipComponent) as e:
        sys_fs_cgroup_memory_tasks_number_data_datasource(broker)
    assert 'SkipComponent' in str(e)



import doctest

from insights.parsers import grubenv
from insights.tests.parsers import skip_exception_check
from insights.tests import context_wrap


GRUBENV_WITH_TUNED_PARAMS = """
saved_entry=295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64
kernelopts=root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200 
boot_success=0
boot_indeterminate=2
tuned_params=transparent_hugepages=never
tuned_initrd=
""".strip()  # noqa

GRUBENV_WITHOUT_TUNED_PARAMS = """
saved_entry=295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64
kernelopts=root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200 
boot_success=0
boot_indeterminate=2
""".strip()  # noqa

GRUBENV_RHEL7 = """
saved_entry=Red Hat Enterprise Linux Server (3.10.0-1127.el7.x86_64) 7.8 (Maipo)
""".strip()  # noqa

GRUBEDITENVLIST_ERROR = """
grub2-editenv: error: invalid environment block.
""".strip()


def test_doc_examples():
    env = {
        'grubenv': grubenv.GrubEnv(context_wrap(GRUBENV_WITH_TUNED_PARAMS))
    }
    failed, total = doctest.testmod(grubenv, globs=env)
    assert failed == 0


def test_with_tuned_params():
    results = grubenv.GrubEnv(context_wrap(GRUBENV_WITH_TUNED_PARAMS))
    assert results is not None
    assert results.has_kernelopts
    assert results.has_tuned_params
    assert results.kernelopts == "root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200"  # noqa
    assert results.tuned_params == "transparent_hugepages=never"
    assert results['saved_entry'] == "295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64"
    assert results['boot_success'] == "0"
    assert results['boot_indeterminate'] == "2"


def test_without_tuned_params():
    results = grubenv.GrubEnv(context_wrap(GRUBENV_WITHOUT_TUNED_PARAMS))
    assert results is not None
    assert results.has_kernelopts
    assert not results.has_tuned_params
    assert results.kernelopts == "root=/dev/mapper/root_vg-lv_root ro crashkernel=auto resume=/dev/mapper/root_vg-lv_swap rd.lvm.lv=root_vg/lv_root rd.lvm.lv=root_vg/lv_swap console=tty0 console=ttyS0,115200"  # noqa
    assert results.tuned_params == ""
    assert results['saved_entry'] == "295e1ba1696e4fad9e062f096f92d147-4.18.0-305.el8.x86_64"
    assert results['boot_success'] == "0"
    assert results['boot_indeterminate'] == "2"


def test_r7():
    results = grubenv.GrubEnv(context_wrap(GRUBENV_RHEL7))
    assert results is not None
    assert not results.has_kernelopts
    assert not results.has_tuned_params
    assert results.kernelopts == ""
    assert results.tuned_params == ""
    assert results['saved_entry'] == "Red Hat Enterprise Linux Server (3.10.0-1127.el7.x86_64) 7.8 (Maipo)"


def test_skip():
    skip_exception_check(grubenv.GrubEnv, output_str="# test")
    skip_exception_check(grubenv.GrubEnv)


def test_error():
    results = grubenv.GrubEnv(context_wrap(GRUBEDITENVLIST_ERROR))
    assert results is not None
    assert results.errors == ["grub2-editenv: error: invalid environment block."]



import doctest

from insights.parsers import neutron_plugin
from insights.tests import context_wrap

neutron_plugin_content = """
[ml2]
# (ListOpt) List of network type driver entrypoints to be loaded from
# the neutron.ml2.type_drivers namespace.
#
# type_drivers = local,flat,vlan,gre,vxlan,geneve
type_drivers = local,flat,vlan,gre,vxlan
# Example: type_drivers = flat,vlan,gre,vxlan,geneve

# (ListOpt) Ordered list of network_types to allocate as tenant
# networks. The default value 'local' is useful for single-box testing
# but provides no connectivity between hosts.
#
# tenant_network_types = local
tenant_network_types = local,flat,vlan,gre,vxlan
# Example: tenant_network_types = vlan,gre,vxlan,geneve


# (ListOpt) Ordered list of networking mechanism driver entrypoints
# to be loaded from the neutron.ml2.mechanism_drivers namespace.
# mechanism_drivers =
mechanism_drivers =openvswitch,linuxbridge
# Example: mechanism_drivers = openvswitch,mlnx
# Example: mechanism_drivers = arista
# Example: mechanism_drivers = openvswitch,cisco_nexus,logger
# Example: mechanism_drivers = openvswitch,brocade
# Example: mechanism_drivers = linuxbridge,brocade

# (ListOpt) Ordered list of extension driver entrypoints
# to be loaded from the neutron.ml2.extension_drivers namespace.
# extension_drivers =
extension_drivers =
# Example: extension_drivers = anewextensiondriver

# =========== items for MTU selection and advertisement =============
# (IntOpt) Path MTU.  The maximum permissible size of an unfragmented
# packet travelling from and to addresses where encapsulated Neutron
# traffic is sent.  Drivers calculate maximum viable MTU for
# validating tenant requests based on this value (typically,
# path_mtu - max encap header size).  If <=0, the path MTU is
# indeterminate and no calculation takes place.
# path_mtu = 0
path_mtu = 0

# (IntOpt) Segment MTU.  The maximum permissible size of an
# unfragmented packet travelling a L2 network segment.  If <=0,
# the segment MTU is indeterminate and no calculation takes place.
# segment_mtu = 0
# (ListOpt) Physical network MTUs.  List of mappings of physical
# network to MTU value.  The format of the mapping is
# <physnet>:<mtu val>.  This mapping allows specifying a
# physical network MTU value that differs from the default
# segment_mtu value.
# physical_network_mtus =
# Example: physical_network_mtus = physnet1:1550, physnet2:1500
# ======== end of items for MTU selection and advertisement =========

# (StrOpt) Default network type for external networks when no provider
# attributes are specified. By default it is None, which means that if
# provider attributes are not specified while creating external networks
# then they will have the same type as tenant networks.
# Allowed values for external_network_type config option depend on the
# network type values configured in type_drivers config option.
# external_network_type =
# Example: external_network_type = local

[ml2_type_flat]
# (ListOpt) List of physical_network names with which flat networks
# can be created. Use * to allow flat networks with arbitrary
# physical_network names.
#
# flat_networks =
flat_networks =*
# Example:flat_networks = physnet1,physnet2
# Example:flat_networks = *

[ml2_type_vlan]
# (ListOpt) List of <physical_network>[:<vlan_min>:<vlan_max>] tuples
# specifying physical_network names usable for VLAN provider and
# tenant networks, as well as ranges of VLAN tags on each
# physical_network available for allocation as tenant networks.
#
# network_vlan_ranges =
network_vlan_ranges =physnet1:1000:2999
# Example: network_vlan_ranges = physnet1:1000:2999,physnet2

[ml2_type_gre]
# (ListOpt) Comma-separated list of <tun_min>:<tun_max> tuples enumerating ranges of GRE tunnel IDs that are available for tenant network allocation
# tunnel_id_ranges =
tunnel_id_ranges =20:100
[ml2_type_vxlan]
# (ListOpt) Comma-separated list of <vni_min>:<vni_max> tuples enumerating
# ranges of VXLAN VNI IDs that are available for tenant network allocation.
#
# vni_ranges =
vni_ranges =10:100

# (StrOpt) Multicast group for the VXLAN interface. When configured, will
# enable sending all broadcast traffic to this multicast group. When left
# unconfigured, will disable multicast VXLAN mode.
#
# vxlan_group =
vxlan_group =224.0.0.1
# Example: vxlan_group = 239.1.1.1

[ml2_type_geneve]
# (ListOpt) Comma-separated list of <vni_min>:<vni_max> tuples enumerating
# ranges of Geneve VNI IDs that are available for tenant network allocation.
#
# vni_ranges =

# (IntOpt) Geneve encapsulation header size is dynamic, this
# value is used to calculate the maximum MTU for the driver.
# this is the sum of the sizes of the outer ETH+IP+UDP+GENEVE
# header sizes.
# The default size for this field is 50, which is the size of the
# Geneve header without any additional option headers
#
# max_header_size =
# Example: max_header_size = 50 (Geneve headers with no additional options)

[securitygroup]
# Controls if neutron security group is enabled or not.
# It should be false when you use nova security group.
# enable_security_group = True
enable_security_group = True

# Use ipset to speed-up the iptables security groups. Enabling ipset support
# requires that ipset is installed on L2 agent node.
# enable_ipset = True
"""


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        neutron_plugin,
        globs={'conf': neutron_plugin.NeutronPlugin(context_wrap(neutron_plugin_content))}
    )
    assert failed_count == 0


def test_neutron_ini():
    result = neutron_plugin.NeutronPlugin(context_wrap(neutron_plugin_content))
    assert result.get("ml2_type_flat", "flat_networks") == "*"
    assert result.get("securitygroup", "enable_security_group") == "True"
    assert result.get("ml2_type_gre", "tunnel_id_ranges") == "20:100"
    assert result.get("ml2_type_vxlan", "vni_ranges") == "10:100"



"""
JbossDomainServerLog - file ``$JBOSS_SERVER_LOG_DIR/server.log*``
=================================================================

Parser for the JBoss Domain Server Log File.
"""

from .. import LogFileOutput, parser
import re
from datetime import time
from insights.specs import Specs


class JbossDomainLog(LogFileOutput):
    """
    Read JBoss domain log file.
    """
    time_format = '%H:%M:%S'
    _line_re = re.compile(r'^(?P<timestamp>\d+:\d+:\d+)(,\d+)(?P<messages>.*)$')

    def get_after(self, timestamp, s=None):
        """
        Find all the (available) logs that are after the given time stamp.

        If `s` is not supplied, then all lines are used.  Otherwise, only the
        lines contain the `s` are used.  `s` can be either a single string or a
        strings list. For list, all keywords in the list must be found in the
        line.

        .. note::
            The time stamp is time type instead of usual datetime type. If
            a time stamp is not found on the line between square brackets, then
            it is treated as a continuation of the previous line and is only
            included if the previous line's timestamp is greater than the
            timestamp given.  Because continuation lines are only included if a
            previous line has matched, this means that searching in logs that do
            not have a time stamp produces no lines.

        Parameters:
            timestamp(time): log lines after this time are returned.
            s(str or list): one or more strings to search for.
                If not supplied, all available lines are searched.

        Yields:
            Log lines with time stamps after the given time.

        Raises:
            TypeError: The ``timestamp`` should be in `time` type, otherwise a
                `TypeError` will be raised.
        """
        if not isinstance(timestamp, time):
            raise TypeError(
                "get_after needs a time type timestamp, but get '{c}'".format(
                    c=timestamp)
            )
        including_lines = False
        search_by_expression = self._valid_search(s)
        for line in self.lines:
            # If `s` is not None, keywords must be found in the line
            if s and not search_by_expression(line):
                continue
            # Otherwise, search all lines
            match = self._line_re.search(line)
            if match and match.group('timestamp'):
                # Get logtimestamp and compare to given timestamp
                l_hh, l_mm, l_ss = match.group('timestamp').split(":")
                logstamp = time(int(l_hh), int(l_mm), int(l_ss))
                if logstamp >= timestamp:
                    including_lines = True
                    yield self._parse_line(line)
                else:
                    including_lines = False
            else:
                # If we're including lines, add this continuation line
                if including_lines:
                    yield self._parse_line(line)


@parser(Specs.jboss_domain_server_log)
class JbossDomainServerLog(JbossDomainLog):
    """
    Read JBoss domain server log file.

    Sample input::


        16:22:57,476 INFO  [org.xnio] (MSC service thread 1-12) XNIO Version 3.0.14.GA-redhat-1
        16:22:57,480 INFO  [org.xnio.nio] (MSC service thread 1-12) XNIO NIO Implementation Version 3.0.14.GA-redhat-1
        16:22:57,495 INFO  [org.jboss.remoting] (MSC service thread 1-12) JBoss Remoting version 3.3.5.Final-redhat-1
        16:23:03,881 INFO  [org.jboss.as.controller.management-deprecated] (ServerService Thread Pool -- 23) JBAS014627: Attribute 'enabled' in the resource at address '/subsystem=datasources/data-source=ExampleDS' is deprecated, and may be removed in future version. See the attribute description in the output of the read-resource-description operation to learn more about the deprecation.
        16:23:03,958 INFO  [org.jboss.as.security] (ServerService Thread Pool -- 37) JBAS013371: Activating Security Subsystem


    Examples:
        >>> type(log)
        <class 'insights.parsers.jboss_domain_log.JbossDomainServerLog'>
        >>> log.file_path
        '/home/test/jboss/machine2/domain/servers/server-one/log/server.log'
        >>> log.file_name
        'server.log'
        >>> error_msgs = log.get('3.0.14.GA-redhat-1')
        >>> error_msgs[0]['raw_message']
        '16:22:57,476 INFO  [org.xnio] (MSC service thread 1-12) XNIO Version 3.0.14.GA-redhat-1'
        >>> 'Activating Security Subsystem' in log
        True
        >>> from datetime import time
        >>> list(log.get_after(time(16, 23, 3)))[1]['raw_message']
        '16:23:03,958 INFO  [org.jboss.as.security] (ServerService Thread Pool -- 37) JBAS013371: Activating Security Subsystem'
    """
    pass


@parser(Specs.jboss_standalone_server_log)
class JbossStandaloneServerLog(JbossDomainLog):
    """
    Read JBoss standalone server log file.

    Sample input::


        2018-07-17 10:58:44,606 INFO  [org.jboss.modules] (main) JBoss Modules version 1.6.0.Final-redhat-1
        2018-07-17 10:58:44,911 INFO  [org.jboss.msc] (main) JBoss MSC version 1.2.7.SP1-redhat-1
        2018-07-17 10:58:45,032 INFO  [org.jboss.as] (MSC service thread 1-7) WFLYSRV0049: JBoss EAP 7.1.0.GA (WildFly Core 3.0.10.Final-redhat-1) starting
        2018-07-17 10:58:45,033 DEBUG [org.jboss.as.config] (MSC service thread 1-7) Configured system properties:
            [Standalone] =
            awt.toolkit = sun.awt.X11.XToolkit
            file.encoding = UTF-8
            file.encoding.pkg = sun.io
            file.separator = /
            java.awt.graphicsenv = sun.awt.X11GraphicsEnvironment
            java.awt.headless = true
            java.awt.printerjob = sun.print.PSPrinterJob
            java.class.path = /opt/jboss-eap-7.1/jboss-modules.jar
            java.class.version = 52.0
            java.endorsed.dirs = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/endorsed
            java.ext.dirs = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/ext:/usr/java/packages/lib/ext
            java.home = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre
            java.io.tmpdir = /tmp
            java.library.path = /usr/java/packages/lib/amd64:/usr/lib64:/lib64:/lib:/usr/lib
            java.net.preferIPv4Stack = true
            java.runtime.name = OpenJDK Runtime Environment
            java.runtime.version = 1.8.0_111-b16
            java.specification.name = Java Platform API Specification
            java.specification.vendor = Oracle Corporation
            java.specification.version = 1.8
            java.util.logging.manager = org.jboss.logmanager.LogManager
            java.vendor = Oracle Corporation
            java.vendor.url = http://java.oracle.com/
            java.vendor.url.bug = http://bugreport.sun.com/bugreport/
            java.version = 1.8.0_111
            java.vm.info = mixed mode
            java.vm.name = OpenJDK 64-Bit Server VM
            java.vm.specification.name = Java Virtual Machine Specification
            java.vm.specification.vendor = Oracle Corporation
            java.vm.specification.version = 1.8
            java.vm.vendor = Oracle Corporation
            java.vm.version = 25.111-b16
            javax.management.builder.initial = org.jboss.as.jmx.PluggableMBeanServerBuilder
            javax.xml.datatype.DatatypeFactory = __redirected.__DatatypeFactory
            javax.xml.parsers.DocumentBuilderFactory = __redirected.__DocumentBuilderFactory
            javax.xml.parsers.SAXParserFactory = __redirected.__SAXParserFactory
            javax.xml.stream.XMLEventFactory = __redirected.__XMLEventFactory
            javax.xml.stream.XMLInputFactory = __redirected.__XMLInputFactory
            javax.xml.stream.XMLOutputFactory = __redirected.__XMLOutputFactory
            javax.xml.transform.TransformerFactory = __redirected.__TransformerFactory
            javax.xml.validation.SchemaFactory:http://www.w3.org/2001/XMLSchema = __redirected.__SchemaFactory
            javax.xml.xpath.XPathFactory:http://java.sun.com/jaxp/xpath/dom = __redirected.__XPathFactory
            jboss.home.dir = /opt/jboss-eap-7.1
            jboss.host.name = mylinux
            jboss.modules.dir = /opt/jboss-eap-7.1/modules
            jboss.modules.system.pkgs = org.jboss.byteman
            jboss.node.name = mylinux
            jboss.qualified.host.name = mylinux
            jboss.server.base.dir = /opt/jboss-eap-7.1/standalone
            jboss.server.config.dir = /opt/jboss-eap-7.1/standalone/configuration
            jboss.server.data.dir = /opt/jboss-eap-7.1/standalone/data
            jboss.server.deploy.dir = /opt/jboss-eap-7.1/standalone/data/content
            jboss.server.log.dir = /opt/jboss-eap-7.1/standalone/log
            jboss.server.name = mylinux
            jboss.server.persist.config = true
            jboss.server.temp.dir = /opt/jboss-eap-7.1/standalone/tmp
            line.separator =

            logging.configuration = file:/opt/jboss-eap-7.1/standalone/configuration/logging.properties
            module.path = /opt/jboss-eap-7.1/modules
            org.jboss.boot.log.file = /opt/jboss-eap-7.1/standalone/log/server.log
            org.jboss.resolver.warning = true
            org.xml.sax.driver = __redirected.__XMLReaderFactory
            os.arch = amd64
            os.name = Linux
            os.version = 4.8.13-100.fc23.x86_64
            path.separator = :
            sun.arch.data.model = 64
            sun.boot.class.path = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/resources.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/rt.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/sunrsasign.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/jsse.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/jce.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/charsets.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/jfr.jar:/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/classes
            sun.boot.library.path = /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.111-1.b16.fc23.x86_64/jre/lib/amd64
            sun.cpu.endian = little
            sun.cpu.isalist =
            sun.desktop = gnome
            sun.io.unicode.encoding = UnicodeLittle
            sun.java.command = /opt/jboss-eap-7.1/jboss-modules.jar -mp /opt/jboss-eap-7.1/modules org.jboss.as.standalone -Djboss.home.dir=/opt/jboss-eap-7.1 -Djboss.server.base.dir=/opt/jboss-eap-7.1/standalone --server-config=standalone-ha.xml
            sun.java.launcher = SUN_STANDARD

    Examples:
        >>> type(standalone_log)
        <class 'insights.parsers.jboss_domain_log.JbossStandaloneServerLog'>
        >>> standalone_log.file_path
        '/JBOSS_HOME/standalone/log/server.log'
        >>> standalone_log.file_name
        'server.log'
        >>> len(standalone_log.get("sun.java.command ="))
        1
    """
    pass



"""
NFS exports configuration
=========================

NFSExports and NFSExportsD provide a parsed output of the content of an exports
file as defined in ``man exports(5)``.  The content is parsed into a
dictionary, where the key is the export path and the value is another
dictionary, where the key is the hostname and the value is the option list,
parsed into an actual list.

The default (``"-"``) hostname is not specially handled, nor are wildcards.

If export paths are defined multiple times in a file, only the first one is
parsed.  All subsequent redefinitions are not parsed and the raw line is added
to the ``ignored_lines`` member.

All raw lines are kept in ``raw_lines``, which is a ``dict`` where the key is
the export path and the value is the stripped raw line.

Parsers included in this module are:

NFSExports - file ``nfs_exports``
---------------------------------

NFSExportsD - files in the ``nfs_exports.d`` directory
------------------------------------------------------

Sample content of the ``/etc/exports`` file::

    /home/utcs/shared/ro                    @group(ro,sync)   ins1.example.com(rw,sync,no_root_squash) ins2.example.com(rw,sync,no_root_squash)
    /home/insights/shared/rw                @group(rw,sync)   ins1.example.com(rw,sync,no_root_squash) ins2.example.com(ro,sync,no_root_squash)
    /home/insights/shared/special/all/mail  @group(rw,sync,no_root_squash)
    /home/insights/ins/special/all/config   @group(ro,sync,no_root_squash)  ins1.example.com(rw,sync,no_root_squash)
    #/home/insights                          ins1.example.com(rw,sync,no_root_squash)
    /home/example                           @group(rw,sync,root_squash) ins1.example.com(rw,sync,no_root_squash) ins2.example.com(rw,sync,no_root_squash)
    # A duplicate host for this exported path
    /home/example                           ins2.example.com(rw,sync,no_root_squash)

Examples:
    >>> type(exports)
    <class 'insights.parsers.nfs_exports.NFSExports'>
    >>> type(exports.data) == type({})
    True
    >>> exports.raw_lines['/home/insights/shared/rw']  # List of lines that define this path
    ['/home/insights/shared/rw                @group(rw,sync)   ins1.example.com(rw,sync,no_root_squash) ins2.example.com(ro,sync,no_root_squash)']
    >>> exports.raw_lines['/home/example']  # Lines are stored even if they contain duplicate hosts
    ['/home/example                           @group(rw,sync,root_squash) ins1.example.com(rw,sync,no_root_squash) ins2.example.com(rw,sync,no_root_squash)', '/home/example                           ins2.example.com(rw,sync,no_root_squash)']
    >>> exports.ignored_exports
    {'/home/example': {'ins2.example.com': ['rw', 'sync', 'no_root_squash']}}
    >>> sorted(list(exports.all_options()))
    ['no_root_squash', 'ro', 'root_squash', 'rw', 'sync']
    >>> sorted(list(exports.export_paths()))
    ['/home/example', '/home/insights/ins/special/all/config', '/home/insights/shared/rw', '/home/insights/shared/special/all/mail', '/home/utcs/shared/ro']
"""

from itertools import chain

from .. import Parser, parser
from . import get_active_lines
from insights.specs import Specs
from insights.util import deprecated


class NFSExportsBase(Parser):
    """
    Class to parse ``/etc/exports`` and ``/etc/exports.d/*.exports``.

    Exports are stored keyed on the path of the export, and then the host
    definition.  The flags are stored as a list.  NFS allows the same path
    to be listed on multiple lines and in multiple files, but an exported
    path can only have one definition for a given host.

    Attributes:
        data (dict): Key is export path, value is a dict, where the key is the
            client host and the value is a list of options.

        ignored_exports (dict): A dictionary of exported paths that have host
            definitions that conflicted with a previous definition.

        ignored_lines (dict): A synonym for the above `ignored_exports`
            dictionary, for historical reasons.

        raw_lines (dict of lists): The list of the raw lines that define each
            exported path, including any lines that may have ignored exports.
    """

    def _parse_host(self, content):
        if "(" in content:
            host, options = content.split("(")
            options = options.rstrip(")").split(",")
        else:
            host, options = content, []
        return host, options

    def _parse_line(self, line):
        split = [i.strip() for i in line.split()]
        path, hosts = split[0], dict(self._parse_host(s) for s in split[1:])
        return path, hosts

    def parse_content(self, content):
        # Exports can be duplicated, but the path-host tuple cannot: the
        # first read will be stored and all later path-host tuples cause
        # `exportfs` to generate a warning when setting up the export.
        self.data = {}
        self.ignored_exports = {}
        self.ignored_lines = self.ignored_exports
        self.raw_lines = {}

        for line in get_active_lines(content):
            path, hosts = self._parse_line(line)
            if path not in self.data:
                # New path, just add the hosts.
                self.data[path] = hosts
                self.raw_lines[path] = [line]
            else:
                # Add to raw lines even if some (or all) hosts are ignored.
                self.raw_lines[path].append(line)
                # Have to check each path-host tuple
                for host, flags in hosts.items():
                    if host not in self.data[path]:
                        # Only add if it doesn't already exist.
                        self.data[path][host] = flags
                    else:
                        if path not in self.ignored_exports:
                            self.ignored_exports[path] = {host: flags}
                        else:
                            self.ignored_exports[path][host] = flags

    def export_paths(self):
        """Returns the set of all export paths as strings"""
        return set(self.data.keys())

    def all_options(self):
        """Returns the set of all options used in all export entries"""
        items = chain.from_iterable(hosts.values() for hosts in self.data.values())
        return set(chain.from_iterable(items))

    def __iter__(self):
        return iter(self.data.items())

    @staticmethod
    def reconstitute(path, d):
        """
        .. warning::
            This function is deprecated.  Please use the `raw_lines` dictionary
            property of the parser instance instead, as this contains the actual
            lines from the exports file.

        'Reconstitute' a line from its parsed value.  The original lines are
        not used for this.  The hosts in d are listed in alphabetical order,
        and the options are listed in the order originally given.

        Arguments:
            path (str): The exported path
            d (dict): The hosts definition of the exported path

        Returns:
            str: A line simulating the definition of that exported path to
            those hosts.
        """
        deprecated(
            NFSExportsBase.reconstitute,
            "Please use the `raw_lines` dictionary property of the parser instance.",
            "3.0.300"
        )
        return "  ".join([path] + ["%s(%s)" % (host, ",".join(options))
                         for host, options in d.items()])


@parser(Specs.nfs_exports)
class NFSExports(NFSExportsBase):
    """Subclass to attach ``nfs_exports`` spec to"""
    pass


@parser(Specs.nfs_exports_d)
class NFSExportsD(NFSExportsBase):
    """Subclass to attach ``nfs_exports.d`` spec to"""
    pass



"""
HasCryptsetupWithTokens, HasCryptsetupWithoutTokens
===================================================

The ``HasCryptsetupWithTokens``/``HasCryptsetupWithoutTokens`` component uses
``InstalledRpms`` parser to determine if cryptsetup package is installed and if
it has tokens support (since version 2.4.0), if not it raises ``SkipComponent``
so that the dependent component will not fire. Can be added as a dependency of
a parser so that the parser only fires if the ``cryptsetup`` dependency and
token support is met.
"""
from insights.core.plugins import component
from insights.parsers.installed_rpms import InstalledRpms, InstalledRpm
from insights.core.dr import SkipComponent


@component(InstalledRpms)
class HasCryptsetupWithTokens(object):
    """The ``HasCryptsetupWithTokens`` component uses ``InstalledRpms`` parser
    to determine if cryptsetup package is installed and if it has tokens
    support (since version 2.4.0), if not it raises ``SkipComponent``

    Raises:
        SkipComponent: When ``cryptsetup`` package is strictly less than 2.4.0,
        or when cryptsetup package is not installed
    """
    def __init__(self, rpms):
        rpm = rpms.get_max("cryptsetup")

        if rpm is None:
            raise SkipComponent("cryptsetup package is not installed")

        if rpm < InstalledRpm("cryptsetup-2.4.0-0"):
            raise SkipComponent("cryptsetup package with token support is not installed")


@component(InstalledRpms)
class HasCryptsetupWithoutTokens(object):
    """The ``HasCryptsetupWithoutTokens`` component uses ``InstalledRpms``
    parser to determine if cryptsetup package is installed and if it does not
    have tokens support (below version 2.4.0), if not it raises
    ``SkipComponent``

    Raises:
        SkipComponent: When ``cryptsetup`` package is at least 2.4.0, or when
        cryptsetup package is not installed
    """
    def __init__(self, rpms):
        rpm = rpms.get_max("cryptsetup")

        if rpm is None:
            raise SkipComponent("cryptsetup package is not installed")

        if rpm >= InstalledRpm("cryptsetup-2.4.0-0"):
            raise SkipComponent("cryptsetup package with token support is installed")



import doctest
from insights.parsers import neutron_l3_agent_log
from insights.parsers.neutron_l3_agent_log import NeutronL3AgentLog
from insights.tests import context_wrap

from datetime import datetime

AGENT_LOG = """
2017-09-17 10:05:06.241 141544 INFO neutron.agent.l3.ha [-] Router 01d51830-0e3e-4100-a891-efd7dbc000b1 transitioned to backup
2017-09-17 10:05:07.828 141544 WARNING neutron.agent.linux.iptables_manager [-] Duplicate iptables rule detected. This may indicate a bug in the the iptables rule generation code. Line: -A neutron-l3-agent-INPUT -p tcp -m tcp --dport 9697 -j DROP
2017-09-17 10:05:07.829 141544 WARNING neutron.agent.linux.iptables_manager [-] Duplicate iptables rule detected. This may indicate a bug in the the iptables rule generation code. Line: -A neutron-l3-agent-INPUT -m mark --mark 0x1/0xffff -j ACCEP
"""


def test_metrics_log():
    log = NeutronL3AgentLog(context_wrap(AGENT_LOG))
    assert len(log.get('INFO')) == 1
    assert 'Duplicate iptables rule detected' in log
    assert len(log.get('Duplicate iptables rule detected')) == 2
    assert len(list(log.get_after(datetime(2017, 2, 17, 19, 36, 38)))) == 3


def test_doc():
    env = {'agent_log': NeutronL3AgentLog(context_wrap(AGENT_LOG, path='/var/log/neutron/l3-agent.log'))}
    failed, total = doctest.testmod(neutron_l3_agent_log, globs=env)
    assert failed == 0
